# -*- coding: utf-8 -*-

from collective.iconifiedcategory.utils import calculate_category_id
from imio.helpers.cache import cleanRamCacheFor
from imio.pm.ws.config import POD_TEMPLATE_ID_PATTERN
from imio.pm.ws.soap.soapview import SOAPView
from imio.pm.ws.tests.WS4PMTestCase import serializeRequest
from imio.pm.ws.tests.WS4PMTestCase import WS4PMTestCase
from imio.pm.ws.WS4PM_client import getItemInfosRequest
from imio.pm.ws.WS4PM_client import getItemInfosResponse
from plone import api
from plone import namedfile
from Products.CMFPlone.utils import safe_unicode
from Products.PloneMeeting.utils import get_annexes
from time import localtime
from ZSI.TCtimes import gDateTime

import base64
import os
import ZSI


class testSOAPGetItemInfos(WS4PMTestCase):
    """
        Tests the soap.getItemInfosRequest and soap.getItemInfosRequest methods
        by accessing the real SOAP service
    """

    def test_ws_getItemInfosRequest(self):
        """
          Test that getting an item with a given UID returns valuable informations
        """
        # by default no item exists
        self.changeUser('pmCreator1')
        self.failUnless(len(self.portal.portal_catalog(portal_type='MeetingItemPga')) == 0)
        # use the SOAP service to create one
        req = self._prepareCreationData()
        newItem, response = self._createItem(req)
        newItemUID = newItem.UID()
        # now an item exists, get informations about it
        req = getItemInfosRequest()
        req._UID = newItemUID
        # Serialize the request so it can be easily tested
        request = serializeRequest(req)
        # This is what the sent enveloppe should looks like
        expected = """<SOAP-ENV:Envelope xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
            """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
            """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
            """xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">""" \
            """<SOAP-ENV:Header></SOAP-ENV:Header>""" \
            """<SOAP-ENV:Body xmlns:ns1="http://ws4pm.imio.be"><ns1:getItemInfosRequest>""" \
            """<UID>%s</UID><showExtraInfos>false</showExtraInfos><showAnnexes>false</showAnnexes>""" \
            """<include_annex_binary>true</include_annex_binary>""" \
            """<showAssembly>false</showAssembly><showTemplates>false</showTemplates>""" \
            """<showEmptyValues>true</showEmptyValues></ns1:getItemInfosRequest>""" \
            """</SOAP-ENV:Body></SOAP-ENV:Envelope>""" % newItemUID
        result = """%s""" % request
        self.assertEqual(expected, result)
        # now really use the SOAP method to get informations about the item
        resp = self._getItemInfos(newItemUID)
        # the item is not in a meeting so the meeting date is 1950-01-01
        expected = """<ns1:getItemInfosResponse xmlns:ns1="http://ws4pm.imio.be" """ \
            """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
            """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
            """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" """ \
            """xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <itemInfo xsi:type="ns1:ItemInfo">
    <UID>{0}</UID>
    <id>my-new-item-title</id>
    <title>My new item title</title>
    <creator>pmCreator1</creator>
    <creation_date>{1}</creation_date>
    <modification_date>{2}</modification_date>
    <category>development</category>
    <description>&lt;p&gt;Description&lt;/p&gt;</description>
    <detailedDescription>&lt;p&gt;Detailed description&lt;/p&gt;</detailedDescription>
    <decision>&lt;p&gt;Décision&lt;/p&gt;</decision>
    <preferredMeeting/>
    <preferred_meeting_date>1950-01-01T00:00:00.006Z</preferred_meeting_date>
    <review_state>itemcreated</review_state>
    <meeting_date>1950-01-01T00:00:00.006Z</meeting_date>
    <absolute_url>http://nohost/plone/Members/pmCreator1/mymeetings/plonegov-assembly/my-new-item-title</absolute_url>
    <externalIdentifier/>
    <extraInfos/>
  </itemInfo>
</ns1:getItemInfosResponse>
""".format(newItemUID,
                gDateTime.get_formatted_content(gDateTime(), localtime(newItem.created())),
                gDateTime.get_formatted_content(gDateTime(), localtime(newItem.modified())))
        self.assertEqual(expected, resp)
        # if the item is in a meeting, the result is a bit different because
        # we have valid informations about the meeting_date
        self.changeUser('pmManager')
        meeting = self._createMeetingWithItems()
        itemInMeeting = meeting.get_items(ordered=True)[0]
        # by default, PloneMeeting creates item without title/description/decision...
        itemInMeeting.setTitle('My new item title')
        itemInMeeting.setDescription('<p>Description</p>', mimetype='text/x-html-safe')
        itemInMeeting.setDecision('<p>Décision</p>', mimetype='text/x-html-safe')
        resp = self._getItemInfos(itemInMeeting.UID())
        meetingDate = gDateTime.get_formatted_content(gDateTime(), meeting.date.utctimetuple())
        expected = """<ns1:getItemInfosResponse xmlns:ns1="http://ws4pm.imio.be" """ \
            """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
            """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
            """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
            """xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <itemInfo xsi:type="ns1:ItemInfo">
    <UID>{0}</UID>
    <id>item-2</id>
    <title>My new item title</title>
    <creator>pmManager</creator>
    <creation_date>{1}</creation_date>
    <modification_date>{2}</modification_date>
    <category>development</category>
    <description>&lt;p&gt;Description&lt;/p&gt;</description>
    <detailedDescription/>
    <decision>&lt;p&gt;Décision&lt;/p&gt;</decision>
    <preferredMeeting/>
    <preferred_meeting_date>1950-01-01T00:00:00.006Z</preferred_meeting_date>
    <review_state>presented</review_state>
    <meeting>{3}</meeting>
    <meeting_date>{4}</meeting_date>
    <absolute_url>http://nohost/plone/Members/pmManager/mymeetings/plonegov-assembly/item-2</absolute_url>
    <externalIdentifier/>
    <extraInfos/>
  </itemInfo>
</ns1:getItemInfosResponse>
""".format(itemInMeeting.UID(),
                gDateTime.get_formatted_content(gDateTime(), localtime(itemInMeeting.created())),
                gDateTime.get_formatted_content(gDateTime(), localtime(itemInMeeting.modified())),
                meeting.UID(),
                meetingDate)
        self.assertEqual(expected, resp)
        # if the item with this UID has not been found (user can not access or item does not exists),
        # an empty response is returned
        # unexisting item UID
        resp = self._getItemInfos('aWrongUID')
        expected = """<ns1:getItemInfosResponse xmlns:ns1="http://ws4pm.imio.be" """ \
            """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
            """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
            """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
            """xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"/>
"""
        self.assertEqual(expected, resp)
        # item UID the logged in user can not access
        self.changeUser('pmReviewer2')
        resp = self._getItemInfos(newItemUID)
        self.assertEqual(expected, resp)

    def test_ws_getItemInfosWithExtraInfosRequest(self):
        """
          Test that getting an item with a given UID and specifying that we want
          extraInfos returns every available informations of the item
        """
        self.changeUser('pmCreator1')
        self.failUnless(len(self.portal.portal_catalog(portal_type='MeetingItemPga')) == 0)
        # prepare data for a default item
        req = self._prepareCreationData()
        # add one annex
        data = {'title': 'My annex 1', 'filename': 'smallTestFile.pdf', 'file': 'smallTestFile.pdf'}
        req._creationData._annexes = [self._prepareAnnexInfo(**data)]
        # create the item
        newItem, reponse = self._createItem(req)
        newItemUID = newItem.UID()
        # get informations about the item, by default 'showExtraInfos' is False
        resp = self._getItemInfos(newItemUID, showExtraInfos=True)
        extraInfosFields = SOAPView(self.portal, req)._getExtraInfosFields(newItem)
        # check that every field considered as extra informations is returned in the response
        for extraInfosField in extraInfosFields:
            self.failUnless(extraInfosField.getName() in resp)

    def test_ws_getItemInfosWithAnnexesRequest(self):
        """
          Test that getting an item with a given UID returns valuable informations and linked annexes
        """
        cfg = self.meetingConfig
        self.changeUser('pmCreator1')
        self.failUnless(len(self.portal.portal_catalog(portal_type='MeetingItemPga')) == 0)
        # prepare data for a default item
        req = self._prepareCreationData()
        # add one annex
        data = {'title': 'My annex 1', 'filename': 'smallTestFile.pdf', 'file': 'smallTestFile.pdf'}
        req._creationData._annexes = [self._prepareAnnexInfo(**data)]
        # create the item
        newItem, reponse = self._createItem(req)
        newItemUID = newItem.UID()
        # get informations about the item, by default 'showAnnexes' is False
        resp = self._getItemInfos(newItemUID)
        expected = """<ns1:getItemInfosResponse xmlns:ns1="http://ws4pm.imio.be" """ \
            """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
            """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
            """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" """ \
            """xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <itemInfo xsi:type="ns1:ItemInfo">
    <UID>{0}</UID>
    <id>{1}</id>
    <title>My new item title</title>
    <creator>pmCreator1</creator>
    <creation_date>{2}</creation_date>
    <modification_date>{3}</modification_date>
    <category>development</category>
    <description>&lt;p&gt;Description&lt;/p&gt;</description>
    <detailedDescription>&lt;p&gt;Detailed description&lt;/p&gt;</detailedDescription>
    <decision>&lt;p&gt;Décision&lt;/p&gt;</decision>
    <preferredMeeting/>
    <preferred_meeting_date>1950-01-01T00:00:00.006Z</preferred_meeting_date>
    <review_state>itemcreated</review_state>
    <meeting_date>1950-01-01T00:00:00.006Z</meeting_date>
    <absolute_url>http://nohost/plone/Members/pmCreator1/mymeetings/plonegov-assembly/{4}</absolute_url>
    <externalIdentifier/>
    <extraInfos/>
  </itemInfo>
</ns1:getItemInfosResponse>
""".format(newItemUID,
                newItem.getId(),
                gDateTime.get_formatted_content(gDateTime(), localtime(newItem.created())),
                gDateTime.get_formatted_content(gDateTime(), localtime(newItem.modified())),
                newItem.getId())
        # annexes are not shown by default
        self.assertEqual(expected, resp)
        # now with 'showAnnexes=True'
        financial_annex_type_id = calculate_category_id(cfg.annexes_types.item_annexes.get('financial-analysis'))
        item_annex_type_id = calculate_category_id(cfg.annexes_types.item_annexes.get('item-annex'))
        resp = self._getItemInfos(newItemUID, showAnnexes=True)
        expected = """<ns1:getItemInfosResponse xmlns:ns1="http://ws4pm.imio.be" """ \
            """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
            """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
            """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
            """xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <itemInfo xsi:type="ns1:ItemInfo">
    <UID>{0}</UID>
    <id>{1}</id>
    <title>My new item title</title>
    <creator>pmCreator1</creator>
    <creation_date>{2}</creation_date>
    <modification_date>{3}</modification_date>
    <category>development</category>
    <description>&lt;p&gt;Description&lt;/p&gt;</description>
    <detailedDescription>&lt;p&gt;Detailed description&lt;/p&gt;</detailedDescription>
    <decision>&lt;p&gt;Décision&lt;/p&gt;</decision>
    <preferredMeeting/>
    <preferred_meeting_date>1950-01-01T00:00:00.006Z</preferred_meeting_date>
    <review_state>itemcreated</review_state>
    <meeting_date>1950-01-01T00:00:00.006Z</meeting_date>
    <absolute_url>http://nohost/plone/Members/pmCreator1/mymeetings/plonegov-assembly/{4}</absolute_url>
    <externalIdentifier/>
    <extraInfos/>
    <annexes xsi:type="ns1:AnnexInfo">
      <id>smalltestfile.pdf</id>
      <title>My annex 1</title>
      <annexTypeId>{5}</annexTypeId>
      <filename>smallTestFile.pdf</filename>
      <file>
{6}</file>
    </annexes>
  </itemInfo>
</ns1:getItemInfosResponse>
""".format(newItemUID,
                newItem.getId(),
                gDateTime.get_formatted_content(gDateTime(), localtime(newItem.created())),
                gDateTime.get_formatted_content(gDateTime(), localtime(newItem.modified())),
                newItem.getId(),
                financial_annex_type_id,
                base64.encodestring(get_annexes(newItem)[0].file.data))
        # one annex is shown
        self.assertEqual(expected, resp)
        # now check with several (2) annexes...
        afile = open(os.path.join(os.path.dirname(__file__),
                                  'mediumTestFile.odt'))
        annex_file = afile.read()
        afile.close()
        api.content.create(
            title='My BeautifulTestFile title',
            type='annex',
            file=namedfile.NamedBlobFile(
                annex_file,
                filename=safe_unicode(u'myBeautifulTestFile.odt')),
            container=newItem,
            content_category=item_annex_type_id,
            to_print=False,
            confidential=False)

        resp = self._getItemInfos(newItemUID, showAnnexes=True)
        expected = """<ns1:getItemInfosResponse xmlns:ns1="http://ws4pm.imio.be" """ \
            """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
            """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
            """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
            """xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <itemInfo xsi:type="ns1:ItemInfo">
    <UID>{0}</UID>
    <id>{1}</id>
    <title>My new item title</title>
    <creator>pmCreator1</creator>
    <creation_date>{2}</creation_date>
    <modification_date>{3}</modification_date>
    <category>development</category>
    <description>&lt;p&gt;Description&lt;/p&gt;</description>
    <detailedDescription>&lt;p&gt;Detailed description&lt;/p&gt;</detailedDescription>
    <decision>&lt;p&gt;Décision&lt;/p&gt;</decision>
    <preferredMeeting/>
    <preferred_meeting_date>1950-01-01T00:00:00.006Z</preferred_meeting_date>
    <review_state>itemcreated</review_state>
    <meeting_date>1950-01-01T00:00:00.006Z</meeting_date>
    <absolute_url>http://nohost/plone/Members/pmCreator1/mymeetings/plonegov-assembly/{4}</absolute_url>
    <externalIdentifier/>
    <extraInfos/>
    <annexes xsi:type="ns1:AnnexInfo">
      <id>{5}</id>
      <title>My annex 1</title>
      <annexTypeId>{6}</annexTypeId>
      <filename>smallTestFile.pdf</filename>
      <file>
{7}</file>
    </annexes>
    <annexes xsi:type="ns1:AnnexInfo">
      <id>{8}</id>
      <title>My BeautifulTestFile title</title>
      <annexTypeId>{9}</annexTypeId>
      <filename>myBeautifulTestFile.odt</filename>
      <file>
{10}</file>
    </annexes>
  </itemInfo>
</ns1:getItemInfosResponse>
""".format(newItemUID,
                newItem.getId(),
                gDateTime.get_formatted_content(gDateTime(), localtime(newItem.created())),
                gDateTime.get_formatted_content(gDateTime(), localtime(newItem.modified())),
                newItem.getId(),
                get_annexes(newItem)[0].id,
                financial_annex_type_id,
                base64.encodestring(get_annexes(newItem)[0].file.data),
                get_annexes(newItem)[1].id,
                item_annex_type_id,
                base64.encodestring(get_annexes(newItem)[1].file.data))
        # 2 annexes are shown
        self.assertEqual(expected, resp)

    def test_ws_getItemInfosWithPODTemplatesRequest(self):
        """
          Test that getting an item with a given UID and specifying that we want
          showTemplates returns informations about generatable POD templates
        """
        # in the PM test profile, some templates are only defined for the plonemeeting-assembly
        self.usedMeetingConfigId = "plonemeeting-assembly"
        self.changeUser('pmCreator1')
        self.failUnless(len(self.portal.portal_catalog(portal_type='MeetingItemPma')) == 0)
        # prepare data for a default item
        req = self._prepareCreationData()
        # remove unuseable catagory
        req._creationData._category = ''
        # create the item
        newItem, reponse = self._createItem(req)
        # get informations about the item, by default 'showTemplates' is False
        resp = self._getItemInfos(newItem.UID(), showTemplates=True, toBeDeserialized=False)
        # we have 1 template
        self.assertEqual(len(resp._itemInfo[0]._templates), 1)
        cfg = self.meetingConfig
        # the returned template correspond to the one present in the 'plonemeeting-assembly' meetingConfig
        self.assertEqual(resp._itemInfo[0]._templates[0]._templateId,
                         POD_TEMPLATE_ID_PATTERN.format(cfg.podtemplates.itemTemplate.getId(),
                                                        cfg.podtemplates.itemTemplate.pod_formats[0]))
        self.assertEqual(resp._itemInfo[0]._templates[0]._templateFilename, u'Item.odt')
        self.assertEqual(resp._itemInfo[0]._templates[0]._templateFormat, 'odt')

    def test_ws_getItemInfosWithReusedPODTemplates(self):
        """
          Test when some returned POD templates are reusing another POD template
          so do not have an odt_file.
        """
        # in the PM test profile, some templates are only defined for the plonemeeting-assembly
        self.usedMeetingConfigId = "plonegov-assembly"
        self.changeUser('pmCreator1')
        item = self.create('MeetingItem')
        # first check that the only returned template is a template rusing another
        viewlet = self._get_viewlet(
            context=item,
            manager_name='plone.belowcontenttitle',
            viewlet_name='document-generation-link')
        templates = viewlet.get_generable_templates()
        self.assertEqual(len(templates), 1)
        self.assertTrue(templates[0].pod_template_to_use)
        self.assertIsNone(templates[0].odt_file)
        # get the reponse
        resp = self._getItemInfos(item.UID(), showTemplates=True, toBeDeserialized=False)
        # we have 1 template
        self.assertEqual(len(resp._itemInfo[0]._templates), 1)
        # templateFilename was taken from template to use
        self.assertEqual(resp._itemInfo[0]._templates[0]._templateFilename, u'Item.odt')
        self.assertEqual(resp._itemInfo[0]._templates[0]._templateFormat, 'odt')

    def test_ws_getItemInfosWithAnnexesTypes(self):
        """
          Test that getting an item with an annex_type return only the annexes
          with the corresponding annexTypeId attribute.
        """
        cfg = self.meetingConfig
        self.changeUser('pmCreator1')
        self.failUnless(len(self.portal.portal_catalog(portal_type='MeetingItemPma')) == 0)
        # prepare data for a default item
        req = self._prepareCreationData()
        # add one annex
        data = {'title': 'My annex 1',
                'filename': 'smallTestFile.pdf',
                'file': 'smallTestFile.pdf'}
        req._creationData._annexes = [self._prepareAnnexInfo(**data)]
        # create the item
        newItem, reponse = self._createItem(req)
        financial_annex_type_id = calculate_category_id(
            cfg.annexes_types.item_annexes.get('financial-analysis'))
        item_annex_type_id = calculate_category_id(
            cfg.annexes_types.item_annexes.get('item-annex'))
        allowed_annexes_types = [financial_annex_type_id, item_annex_type_id]
        # get informations about the item, by default 'allowed_annexes_types' is empty
        resp = self._getItemInfos(newItem.UID(),
                                  showAnnexes=True,
                                  toBeDeserialized=False)
        # we have 1 annex
        self.assertEqual(len(resp._itemInfo[0]._annexes), 1)
        # the returned annex is the one created
        self.assertEqual(resp._itemInfo[0]._annexes[0]._title, 'My annex 1')
        self.assertEqual(resp._itemInfo[0]._annexes[0]._filename, 'smallTestFile.pdf')
        # filter on 'item_annex_type' annex type
        allowed_annexes_types = [financial_annex_type_id]
        resp = self._getItemInfos(newItem.UID(),
                                  showAnnexes=True,
                                  allowed_annexes_types=allowed_annexes_types,
                                  toBeDeserialized=False)
        # we have 1 annex
        self.assertEqual(len(resp._itemInfo[0]._annexes), 1)
        # the returned annex is the one created
        self.assertEqual(resp._itemInfo[0]._annexes[0]._title, 'My annex 1')
        self.assertIn(resp._itemInfo[0]._annexes[0]._annexTypeId, allowed_annexes_types)
        # filter on 'financial_annex_type_id' annex type
        allowed_annexes_types = [item_annex_type_id]
        resp = self._getItemInfos(newItem.UID(),
                                  showAnnexes=True,
                                  allowed_annexes_types=allowed_annexes_types,
                                  toBeDeserialized=False)
        # we have 0 annex
        self.assertEqual(len(resp._itemInfo[0]._annexes), 0)

    def test_ws_getItemInfosWithBinary(self):
        """
          Test that getting an item with include_annex_binary return the annex
          binary file.
        """
        self.changeUser('pmCreator1')
        self.failUnless(len(self.portal.portal_catalog(portal_type='MeetingItemPma')) == 0)
        # prepare data for a default item
        req = self._prepareCreationData()
        # add one annex
        data = {'title': 'My annex 1', 'filename': 'smallTestFile.pdf', 'file': 'smallTestFile.pdf'}
        req._creationData._annexes = [self._prepareAnnexInfo(**data)]
        # create the item
        newItem, reponse = self._createItem(req)
        # get informations about the item, by default include_annex_binary is True
        resp = self._getItemInfos(newItem.UID(), showAnnexes=True, toBeDeserialized=False)
        # we have 1 annex
        self.assertEqual(len(resp._itemInfo[0]._annexes), 1)
        # the returned annex is the one created
        self.assertEqual(resp._itemInfo[0]._annexes[0]._title, 'My annex 1')
        # file content is preserved correctly
        annex_file = open(os.path.join(os.path.dirname(__file__), data.get('file')))
        self.assertEqual(resp._itemInfo[0]._annexes[0]._file, annex_file.read())
        # get informations about the item, set include_annex_binary to False
        resp = self._getItemInfos(newItem.UID(),
                                  showAnnexes=True,
                                  include_annex_binary=False,
                                  toBeDeserialized=False)
        # we have 1 annex
        self.assertEqual(len(resp._itemInfo[0]._annexes), 1)
        # the returned annex is the one created
        self.assertEqual(resp._itemInfo[0]._annexes[0]._title, 'My annex 1')
        # attribute _file of the annex should be empty
        self.assertFalse(resp._itemInfo[0]._annexes[0]._file)

    def test_ws_getItemInfosShowEmptyValues(self):
        """
          Parameter showEmptyValues=False will remove empty values from returned result.
        """
        self.changeUser('pmCreator1')
        item = self.create('MeetingItem')
        resp = self._getItemInfos(item.UID(), toBeDeserialized=False)
        # empty values are returned
        self.assertTrue('_decision' in resp._itemInfo[0].__dict__)
        self.assertEqual(resp._itemInfo[0]._decision, '')
        resp = self._getItemInfos(item.UID(), showEmptyValues=False, toBeDeserialized=False)
        # empty values are no more returned
        self.assertFalse('_decision' in resp._itemInfo[0].__dict__)
        self.assertEqual(
            len(resp._itemInfo[0].__dict__),
            len([k for k, v in resp._itemInfo[0].__dict__.items() if v]))

    def test_ws_getItemInfosInTheNameOf(self):
        """
          Test that getting an item inTheNameOf antother user works
          Create an item by 'pmCreator1', member of the 'developers' group
          Item will be viewable :
          - by 'pmManager'
          - while getting informations in the name of 'pmCreator1'
          Item will NOT be viewable :
          - while getting informations in the name of 'pmCreator2'
            that is not in the 'developers' group
        """
        # create an item by 'pmCreator1'
        self.changeUser('pmCreator1')
        item = self.create('MeetingItem')
        # check first a working example the degrades it...
        req = getItemInfosRequest()
        req._inTheNameOf = None
        req._UID = item.UID()
        responseHolder = getItemInfosResponse()
        # 'pmCreator1' can get infos about the item
        result = SOAPView(self.portal, req).getItemInfosRequest(req, responseHolder)
        self.assertTrue(result._itemInfo[0].UID == item.UID())
        # now begin, we need to be a 'MeetingManager' or 'Manager' to
        # getItemInfos(inTheNameOf)
        req._inTheNameOf = 'pmCreator1'
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).getItemInfosRequest(req, responseHolder)
        self.assertEqual(
            cm.exception.string,
            "You need to be 'Manager' or 'MeetingManager' to get item informations 'inTheNameOf'!")
        # now has a 'MeetingManager'
        self.changeUser('pmManager')
        # a MeetingManager can get informations inTheNameOf 'pmCreator1'
        # and it will return relevant result as 'pmCreator1' can see the item
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        result = SOAPView(self.portal, req).getItemInfosRequest(req, responseHolder)
        self.assertTrue(result._itemInfo[0].UID == item.UID())
        # as we switch user while using inTheNameOf, make sure we have
        # falled back to original user
        self.assertTrue(self.portal.portal_membership.getAuthenticatedMember().getId() == 'pmManager')
        # as 'pmCreator2', we can not get item informations
        req._inTheNameOf = 'pmCreator2'
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        result = SOAPView(self.portal, req).getItemInfosRequest(req, responseHolder)
        self.assertTrue(result._itemInfo == [])
        # now for an unexisting inTheNameOf userid
        req._inTheNameOf = 'unexistingUserId'
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).getItemInfosRequest(req, responseHolder)
        self.assertEqual(
            cm.exception.string,
            "Trying to get item informations 'inTheNameOf' an unexisting user 'unexistingUserId'!")
        self._check_after_inTheNameOf()

    def test_ws_getItemInfosWithShowAssembly(self):
        """When showAssembly=True, assembly is returned in a text form
           when using assembly fields or contacts."""
        self.changeUser('pmManager')
        # item out of a meeting
        item = self.create('MeetingItem')
        item_uid = item.UID()
        resp = self._getItemInfos(item_uid, showAssembly=True, toBeDeserialized=False)
        self.assertIsNone(resp.ItemInfo[0]._item_assembly)

        # item in a meeting
        meeting = self._createMeetingWithItems()
        item = meeting.get_items(ordered=True)[0]
        item_uid = item.UID()
        # showAssembly=False
        resp = self._getItemInfos(item_uid, showAssembly=False, toBeDeserialized=False)
        self.assertIsNone(resp.ItemInfo[0]._item_assembly)
        # itemAssembly
        resp = self._getItemInfos(item_uid, showAssembly=True, toBeDeserialized=False)
        self.assertEqual(resp.ItemInfo[0]._item_assembly,
                         'itemAssembly|<p>Bill Gates, Steve Jobs</p>|itemAssemblyExcused||'
                         'itemAssemblyAbsents||itemAssemblyGuests|')
        item.setItemAssembly('Local assembly')
        item.setItemAssemblyAbsents('Local assembly absents')
        item.setItemAssemblyExcused('Local assembly excused')
        item.setItemAssemblyGuests('Local assembly guests')
        resp = self._getItemInfos(item_uid, showAssembly=True, toBeDeserialized=False)
        self.assertEqual(resp.ItemInfo[0]._item_assembly,
                         'itemAssembly|<p>Local assembly</p>|'
                         'itemAssemblyExcused|<p>Local assembly excused</p>|'
                         'itemAssemblyAbsents|<p>Local assembly absents</p>|'
                         'itemAssemblyGuests|<p>Local assembly guests</p>')
        # contacts
        cfg = self.meetingConfig
        cfg.setUsedMeetingAttributes(('attendees', 'excused', 'absents', 'signatories', ))
        ordered_contacts = cfg.getField('orderedContacts').Vocabulary(cfg).keys()
        cfg.setOrderedContacts(ordered_contacts)
        meeting = self._createMeetingWithItems()
        item = meeting.get_items(ordered=True)[0]
        item_uid = item.UID()
        resp = self._getItemInfos(item_uid, showAssembly=False, toBeDeserialized=False)
        self.assertIsNone(resp.ItemInfo[0]._item_assembly)
        resp = self._getItemInfos(item_uid, showAssembly=True, toBeDeserialized=False)
        self.assertEqual(resp.ItemInfo[0]._item_assembly,
                         u'Attendees|Monsieur Person1FirstName Person1LastName, Assembly member 1\n'
                         u'Monsieur Person2FirstName Person2LastName, Assembly member 2\n'
                         u'Monsieur Person3FirstName Person3LastName, Assembly member 3\n'
                         u'Monsieur Person4FirstName Person4LastName, Assembly member 4 & 5|'
                         u'Absents||Excused||itemAssemblyGuests|')
        # define absent on item
        item_attendees = item.get_attendees()
        meeting.item_absents[item_uid] = [item_attendees[0], item_attendees[2]]
        resp = self._getItemInfos(item_uid, showAssembly=True, toBeDeserialized=False)
        self.assertEqual(resp.ItemInfo[0]._item_assembly,
                         u'Attendees|Monsieur Person2FirstName Person2LastName, Assembly member 2\n'
                         u'Monsieur Person4FirstName Person4LastName, Assembly member 4 & 5|'
                         u'Absents|Monsieur Person1FirstName Person1LastName, Assembly member 1\n'
                         u'Monsieur Person3FirstName Person3LastName, Assembly member 3|'
                         u'Excused||itemAssemblyGuests|')

    def test_ws_getSingleItemInfos(self):
        """The getSingleItemInfos method behaves like getItemInfos
            but returns one single item infos instead list of ItemInfo instances."""
        self.changeUser('pmManager')
        # item out of a meeting
        item = self.create('MeetingItem')
        item_uid = item.UID()
        # informations returned by getItemInfos and getSingleItemInfos are equal
        resp_getiteminfos = self._getItemInfos(item_uid, toBeDeserialized=False)
        resp_getsimpleiteminfos = self._getItemInfos(item_uid, toBeDeserialized=False, useSingleItemInfos=True)
        self.assertEqual(resp_getiteminfos._itemInfo[0].__dict__,
                         resp_getsimpleiteminfos.__dict__)
        # as well when using showEmptyValues=False
        resp_getnotemptyiteminfos = self._getItemInfos(item_uid,
                                                       showEmptyValues=False,
                                                       toBeDeserialized=False)
        resp_getnotemptysimpleiteminfos = self._getItemInfos(item_uid,
                                                             showEmptyValues=False,
                                                             toBeDeserialized=False,
                                                             useSingleItemInfos=True)
        self.assertEqual(resp_getnotemptyiteminfos._itemInfo[0].__dict__,
                         resp_getnotemptysimpleiteminfos.__dict__)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and
    # we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testSOAPGetItemInfos, prefix='test_ws_'))
    return suite



# -*- coding: utf-8 -*-

from plone.transformchain.transformer import Transformer
from z3c.soap.interfaces import ISOAPRequest


class SoapAwareTransformer(Transformer):
    """Do not apply transform on SOAP requests."""

    def __call__(self, request, result, encoding):
        # Don't transform SOAP requests
        if ISOAPRequest.providedBy(request):
            return None

        return super(SoapAwareTransformer, self).__call__(request, result, encoding)



# -*- coding: utf-8 -*-
#
# File: test_getitemtemplate.py
#
# Copyright (c) 2013 by Imio.be
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

from imio.helpers.cache import cleanRamCacheFor
from imio.pm.ws.soap.soapview import SOAPView
from imio.pm.ws.tests.WS4PMTestCase import WS4PMTestCase
from imio.pm.ws.WS4PM_client import getItemTemplateRequest
from imio.pm.ws.WS4PM_client import getItemTemplateResponse

import os
import zipfile
import ZSI


class testSOAPGetItemTemplate(WS4PMTestCase):
    """
        Tests the soap.getItemTemplateRequest method by accessing the real SOAP service
    """

    def setUp(self):
        """ """
        WS4PMTestCase.setUp(self)
        # in the PM test profile, some templates are only defined for the plonemeeting-assembly
        self.usedMeetingConfigId = "plonemeeting-assembly"

    def test_ws_getItemTemplateRequest(self):
        """
          Test that getting an item with a given UID and specifying that we want
          showTemplates returns informations about generatable POD templates
        """
        self.changeUser('pmCreator1')
        self.failUnless(len(self.portal.portal_catalog(portal_type='MeetingItemPma')) == 0)
        # prepare data for a default item
        req = self._prepareCreationData()
        # remove unuseable catagory
        req._creationData._category = ''
        # create the item
        newItem, reponse = self._createItem(req)
        # if the user can not access the item, a ZSI Fault is raised
        req = getItemTemplateRequest()
        newItemUID = newItem.UID()
        req._itemUID = newItemUID
        responseHolder = getItemTemplateResponse()
        self.changeUser('pmCreator2')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).getItemTemplateRequest(req, responseHolder)
        self.assertEquals(cm.exception.string, 'You can not access this item!')
        # if we try to use another template, than one available for this item, a ZSI Fault is raised
        self.changeUser('pmCreator1')
        cfg = self.tool.get(self.usedMeetingConfigId)
        wrongTemplate = cfg.podtemplates.agendaTemplate
        req._templateId = '%s__format__%s' % (wrongTemplate.getId(), 'odt')
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).getItemTemplateRequest(req, responseHolder)
        self.assertEquals(cm.exception.string, 'You can not access this template!')
        # if everything is correct, we receive the rendered template
        req._templateId = '{0}__format__{1}'.format(cfg.podtemplates.itemTemplate.getId(), 'odt')
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        renderedTemplate = SOAPView(self.portal, req).getItemTemplateRequest(req, responseHolder)
        # check that the rendered file correspond to the newItem's data
        self._isCorrectlyRenderedTemplate(renderedTemplate, newItem)
        # test if PloneMeeting raise a PloneMeetingError
        # for example, trying to generate a PDF when Ooo is not in server mode
        cfg.podtemplates.itemTemplate.pod_formats = ['pdf']
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).getItemTemplateRequest(req, responseHolder)
        self.assertEquals(cm.exception.string,
                          "Exception : Asked output format 'odt' is not available for template 'itemTemplate'!")

    def test_ws_getItemTemplateInTheNameOf(self):
        """
          Test that getting an item template inTheNameOf antother user works
          Create an item by 'pmCreator1', member of the 'developers' group
          Template will be generatable :
          - by 'pmManager'
          - while generating the template inTheNameOf 'pmCreator1'
          Template will NOT be generatable :
          - while generating the template inTheNameOf 'pmCreator2'
            that is not in the 'developers' group
        """
        # create an item by 'pmCreator1'
        self.changeUser('pmCreator1')
        # prepare data for a default item
        req = self._prepareCreationData()
        # remove unuseable catagory
        req._creationData._category = ''
        # create the item
        newItem, reponse = self._createItem(req)
        # prepare data to query the template
        req = getItemTemplateRequest()
        newItemUID = newItem.UID()
        req._itemUID = newItemUID
        cfg = self.meetingConfig
        req._templateId = '%s__format__%s' % (cfg.podtemplates.itemTemplate.getId(), 'odt')
        req._inTheNameOf = 'pmCreator1'
        # the current user can get item template inTheNameOf himself
        responseHolder = getItemTemplateResponse()
        # need to be a 'Manager' or 'MeetingManager'
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).getItemTemplateRequest(req, responseHolder)
        self.assertEquals(cm.exception.string,
                          "You need to be 'Manager' or 'MeetingManager' to get "
                          "a template for an item 'inTheNameOf'!")
        # now as MeetingManager, it works
        self.changeUser('pmManager')
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        renderedTemplate = SOAPView(self.portal, req).getItemTemplateRequest(req, responseHolder)
        # check that the rendered file correspond to the newItem's data
        self._isCorrectlyRenderedTemplate(renderedTemplate, newItem)
        # as we switch user while using inTheNameOf, make sure we have
        # falled back to original user
        self.assertTrue(self.portal.portal_membership.getAuthenticatedMember().getId() == 'pmManager')
        # now inTheNameOf a user that can not access newItem
        req._inTheNameOf = 'pmCreator2'
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).getItemTemplateRequest(req, responseHolder)
        self.assertEquals(cm.exception.string,
                          "You can not access this item!")
        req._inTheNameOf = 'unexistingUserId'
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).getItemTemplateRequest(req, responseHolder)
        self.assertEquals(cm.exception.string,
                          "Trying to create an item 'inTheNameOf' an unexisting user 'unexistingUserId'!")
        self._check_after_inTheNameOf()

    def _isCorrectlyRenderedTemplate(self, renderedTemplate, item):
        """ """
        itemUID = item.UID()
        tmp_file = file('/tmp/%s.zip' % itemUID, 'w')
        tmp_file.write(renderedTemplate._file)
        tmp_file.close()
        zipped_file = zipfile.ZipFile('/tmp/%s.zip' % itemUID)
        self.assertTrue(item.Title() in zipped_file.read('content.xml'))
        os.remove('/tmp/%s.zip' % itemUID)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testSOAPGetItemTemplate, prefix='test_ws_'))
    return suite



##################################################
# file: WS4PM_client.py
# 
# client stubs generated by "ZSI.generate.wsdl2python.WriteServiceModule"
#     /srv/ZSI/bin/wsdl2py dumpedWSDL.txt -o . -b
# 
##################################################

from WS4PM_types import *
import urlparse, types
from ZSI.TCcompound import ComplexType, Struct
from ZSI import client
from ZSI.schema import GED, GTD
import ZSI
from ZSI.generate.pyclass import pyclass_type

# Locator
class WS4PMLocator:
    WS4PMSOAP_address = "http://ws4pm.imio.be"
    def getWS4PMSOAPAddress(self):
        return WS4PMLocator.WS4PMSOAP_address
    def getWS4PMSOAP(self, url=None, **kw):
        return WS4PMSOAPBindingSOAP(url or WS4PMLocator.WS4PMSOAP_address, **kw)

# Methods
class WS4PMSOAPBindingSOAP:
    def __init__(self, url, **kw):
        kw.setdefault("readerclass", None)
        kw.setdefault("writerclass", None)
        # no resource properties
        self.binding = client.Binding(url=url, **kw)
        # no ws-addressing

    # op: testConnection
    def testConnection(self, request, **kw):
        if isinstance(request, testConnectionRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://ws4pm.imio.be/testConnection", **kw)
        # no output wsaction
        response = self.binding.Receive(testConnectionResponse.typecode)
        return response

    # op: checkIsLinked
    def checkIsLinked(self, request, **kw):
        if isinstance(request, checkIsLinkedRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://ws4pm.imio.be/checkIsLinked", **kw)
        # no output wsaction
        response = self.binding.Receive(checkIsLinkedResponse.typecode)
        return response

    # op: getConfigInfos
    def getConfigInfos(self, request, **kw):
        if isinstance(request, getConfigInfosRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://ws4pm.imio.be/getConfigInfos", **kw)
        # no output wsaction
        response = self.binding.Receive(getConfigInfosResponse.typecode)
        return response

    # op: getUserInfos
    def getUserInfos(self, request, **kw):
        if isinstance(request, getUserInfosRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://ws4pm.imio.be/getUserInfos", **kw)
        # no output wsaction
        response = self.binding.Receive(getUserInfosResponse.typecode)
        return response

    # op: getItemInfos
    def getItemInfos(self, request, **kw):
        if isinstance(request, getItemInfosRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://ws4pm.imio.be/getItemInfos", **kw)
        # no output wsaction
        response = self.binding.Receive(getItemInfosResponse.typecode)
        return response

    # op: getSingleItemInfos
    def getSingleItemInfos(self, request, **kw):
        if isinstance(request, getSingleItemInfosRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://ws4pm.imio.be/getSingleItemInfos", **kw)
        # no output wsaction
        response = self.binding.Receive(getSingleItemInfosResponse.typecode)
        return response

    # op: getItemTemplate
    def getItemTemplate(self, request, **kw):
        if isinstance(request, getItemTemplateRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://ws4pm.imio.be/getItemTemplate", **kw)
        # no output wsaction
        response = self.binding.Receive(getItemTemplateResponse.typecode)
        return response

    # op: searchItems
    def searchItems(self, request, **kw):
        if isinstance(request, searchItemsRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://ws4pm.imio.be/searchItems", **kw)
        # no output wsaction
        response = self.binding.Receive(searchItemsResponse.typecode)
        return response

    # op: meetingsAcceptingItems
    def meetingsAcceptingItems(self, request, **kw):
        if isinstance(request, meetingsAcceptingItemsRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://ws4pm.imio.be/meetingsAcceptingItems", **kw)
        # no output wsaction
        response = self.binding.Receive(meetingsAcceptingItemsResponse.typecode)
        return response

    # op: createItem
    def createItem(self, request, **kw):
        if isinstance(request, createItemRequest) is False:
            raise TypeError, "%s incorrect request type" % (request.__class__)
        # no input wsaction
        self.binding.Send(None, None, request, soapaction="http://ws4pm.imio.be/createItem", **kw)
        # no output wsaction
        response = self.binding.Receive(createItemResponse.typecode)
        return response

testConnectionRequest = GED("http://ws4pm.imio.be", "testConnectionRequest").pyclass

testConnectionResponse = GED("http://ws4pm.imio.be", "testConnectionResponse").pyclass

checkIsLinkedRequest = GED("http://ws4pm.imio.be", "checkIsLinkedRequest").pyclass

checkIsLinkedResponse = GED("http://ws4pm.imio.be", "checkIsLinkedResponse").pyclass

getConfigInfosRequest = GED("http://ws4pm.imio.be", "getConfigInfosRequest").pyclass

getConfigInfosResponse = GED("http://ws4pm.imio.be", "getConfigInfosResponse").pyclass

getUserInfosRequest = GED("http://ws4pm.imio.be", "getUserInfosRequest").pyclass

getUserInfosResponse = GED("http://ws4pm.imio.be", "getUserInfosResponse").pyclass

getItemInfosRequest = GED("http://ws4pm.imio.be", "getItemInfosRequest").pyclass

getItemInfosResponse = GED("http://ws4pm.imio.be", "getItemInfosResponse").pyclass

getSingleItemInfosRequest = GED("http://ws4pm.imio.be", "getSingleItemInfosRequest").pyclass

getSingleItemInfosResponse = GED("http://ws4pm.imio.be", "getSingleItemInfosResponse").pyclass

getItemTemplateRequest = GED("http://ws4pm.imio.be", "getItemTemplateRequest").pyclass

getItemTemplateResponse = GED("http://ws4pm.imio.be", "getItemTemplateResponse").pyclass

searchItemsRequest = GED("http://ws4pm.imio.be", "searchItemsRequest").pyclass

searchItemsResponse = GED("http://ws4pm.imio.be", "searchItemsResponse").pyclass

meetingsAcceptingItemsRequest = GED("http://ws4pm.imio.be", "meetingsAcceptingItemsRequest").pyclass

meetingsAcceptingItemsResponse = GED("http://ws4pm.imio.be", "meetingsAcceptingItemsResponse").pyclass

createItemRequest = GED("http://ws4pm.imio.be", "createItemRequest").pyclass

createItemResponse = GED("http://ws4pm.imio.be", "createItemResponse").pyclass



# -*- coding: utf-8 -*-
#
# File: setuphandlers.py
#
# GNU General Public License (GPL)
#

from Products.CMFCore.utils import getToolByName

import logging


logger = logging.getLogger('imio.pm.ws: setuphandlers')


def isNotImioPmWsProfile(context):
    return context.readDataFile("imio_pm_ws_marker.txt") is None


def postInstall(context):
    """Called as at the end of the setup process. """
    # the right place for your custom code
    if isNotImioPmWsProfile(context):
        return

    portal = context.getSite()
    # Update schema of existing MeetingItems to take
    # new fields added by schemaextender
    _updateMeetingItemsSchema(portal)


def _updateMeetingItemsSchema(portal):
    """
      We add a 'externalIdentifier' field using archetypes.schemaextender
      to every objects providing the IMeetingItem interface
    """
    logger.info("Updating the schema of every MeetingItems...")
    at_tool = getToolByName(portal, 'archetype_tool')
    catalog = getToolByName(portal, 'portal_catalog')
    catalog.ZopeFindAndApply(portal,
                             obj_metatypes=('MeetingItem',),
                             search_sub=True,
                             apply_func=at_tool._updateObject)
    logger.info("Done!")



# -*- coding: utf-8 -*-
#
# File: test_getconfiginfos.py
#
# Copyright (c) 2013 by Imio.be
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#


from collective.contact.plonegroup.utils import get_organizations
from imio.helpers.cache import cleanRamCacheFor
from imio.pm.ws.soap.soapview import SOAPView
from imio.pm.ws.tests.WS4PMTestCase import deserialize
from imio.pm.ws.tests.WS4PMTestCase import serializeRequest
from imio.pm.ws.tests.WS4PMTestCase import WS4PMTestCase
from imio.pm.ws.WS4PM_client import getConfigInfosRequest
from imio.pm.ws.WS4PM_client import getConfigInfosResponse

import ZSI


class testSOAPGetConfigInfos(WS4PMTestCase):
    """
        Tests the soap.getConfigInfosRequest method by accessing the real SOAP service
    """

    def test_ws_getConfigInfosRequest(self):
        """
          Test that getting informations about the configuration returns valuable informations
        """
        # any PM user can have these configuration informations
        self.changeUser('pmCreator1')
        req = getConfigInfosRequest()
        # Serialize the request so it can be easily tested
        request = serializeRequest(req)
        # This is what the sent enveloppe should looks like
        expected = """<SOAP-ENV:Envelope xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
                   """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
                   """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
                   """xmlns:xsd="http://www.w3.org/2001/XMLSchema" """ \
                   """xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">""" \
                   """<SOAP-ENV:Header></SOAP-ENV:Header><SOAP-ENV:Body xmlns:ns1="http://ws4pm.imio.be">""" \
                   """<ns1:getConfigInfosRequest><showCategories>false</showCategories></ns1:getConfigInfosRequest>""" \
                   """</SOAP-ENV:Body></SOAP-ENV:Envelope>"""
        result = """%s""" % request
        self.assertEquals(expected, result)
        # now really use the SOAP method to get informations about the configuration
        responseHolder = getConfigInfosResponse()
        response = SOAPView(self.portal, req).getConfigInfosRequest(req, responseHolder)
        resp = deserialize(response)
        # construct the expected result : header + content + footer
        # header
        expected = """<ns1:getConfigInfosResponse xmlns:ns1="http://ws4pm.imio.be" """ \
                   """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
                   """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
                   """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
                   """xmlns:xsd="http://www.w3.org/2001/XMLSchema" """ \
                   """xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">"""
        # _configInfo
        for cfg in self.tool.getActiveConfigs():
            expected += """
  <configInfo xsi:type="ns1:ConfigInfo">
    <UID>%s</UID>
    <id>%s</id>
    <title>%s</title>
    <description>%s</description>%s
  </configInfo>""" % (cfg.UID(),
                      cfg.getId(),
                      cfg.Title(),
                      cfg.Description(),
                      self._getItemPositiveDecidedStatesFromConfig(cfg))
        # _groupInfo
        for grp in get_organizations():
            expected += """
  <groupInfo xsi:type="ns1:GroupInfo">
    <UID>%s</UID>
    <id>%s</id>
    <title>%s</title>
    <description>%s</description>
  </groupInfo>""" % (grp.UID(),
                     grp.getId(),
                     grp.Title(),
                     grp.Description())
        # footer.  Empty description is represented like <description/>
        expected = expected.replace('<description></description>', '<description/>') \
            + "\n</ns1:getConfigInfosResponse>\n"
        self.assertEquals(expected, resp)

        # elements are correctly stored
        cfg = self.tool.getActiveConfigs()[0]
        self.assertEqual(response._configInfo[0]._id, cfg.id)
        self.assertEqual(response._configInfo[0]._title, cfg.Title())

    def test_ws_getConfigInfosItemPositiveDecidedStates(self):
        """This will return the MeetingConfig.itemPositiveDecidedStates."""
        self.changeUser('pmCreator1')
        req = getConfigInfosRequest()
        responseHolder = getConfigInfosResponse()
        response = SOAPView(self.portal, req).getConfigInfosRequest(req, responseHolder)
        cfg = self.tool.getActiveConfigs()[0]
        # we have defined itemPositiveDecidedStates
        self.assertTrue(cfg.getItemPositiveDecidedStates())
        self.assertEqual(response._configInfo[0]._itemPositiveDecidedStates,
                         cfg.getItemPositiveDecidedStates())
        self.assertTrue(deserialize(response))
        # works if no itemPositiveDecidedStates defined
        cfg.getItemPositiveDecidedStates = lambda: ()
        response = SOAPView(self.portal, req).getConfigInfosRequest(req, responseHolder)
        self.assertEqual(response._configInfo[0]._itemPositiveDecidedStates, ())
        self.assertTrue(deserialize(response))

    def test_ws_showCategories(self):
        """
          Test while getting configInfos with categories.
        """
        self.changeUser('pmCreator1')
        req = getConfigInfosRequest()
        req._showCategories = True
        # Serialize the request so it can be easily tested
        responseHolder = getConfigInfosResponse()
        response = SOAPView(self.portal, req).getConfigInfosRequest(req, responseHolder)
        resp = deserialize(response)
        expected = """<ns1:getConfigInfosResponse xmlns:ns1="http://ws4pm.imio.be" """ \
                   """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
                   """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
                   """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
                   """xmlns:xsd="http://www.w3.org/2001/XMLSchema" """ \
                   """xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">"""
        # _configInfo
        for cfg in self.tool.getActiveConfigs():
            expected += """
  <configInfo xsi:type="ns1:ConfigInfo">
    <UID>%s</UID>
    <id>%s</id>
    <title>%s</title>
    <description>%s</description>%s%s
  </configInfo>""" % (cfg.UID(),
                      cfg.getId(),
                      cfg.Title(),
                      cfg.Description(),
                      self._getItemPositiveDecidedStatesFromConfig(cfg),
                      self._getResultCategoriesForConfig(cfg))
        # _groupInfo
        for grp in get_organizations():
            expected += """
  <groupInfo xsi:type="ns1:GroupInfo">
    <UID>%s</UID>
    <id>%s</id>
    <title>%s</title>
    <description>%s</description>
  </groupInfo>""" % (grp.UID(),
                     grp.getId(),
                     grp.Title(),
                     grp.Description())
        # footer.  Empty description is represented like <description/>
        expected = expected.replace('<description></description>', '<description/>') \
            + "\n</ns1:getConfigInfosResponse>\n"
        self.assertEquals(expected, resp)
        # the category 'subproducts' is only available to vendors
        self.assertFalse('<id>subproducts</id>' in resp)

    def test_ws_showCategoriesForUser(self):
        """
          Test while getting configInfos with categories and using userToShowCategoriesFor.
        """
        # first of all, we need to be a Manager/MeetingManager to use userToShowCategoriesFor
        self.changeUser('pmCreator1')
        req = getConfigInfosRequest()
        req._showCategories = True
        req._userToShowCategoriesFor = 'pmCreator1'
        responseHolder = getConfigInfosResponse()
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).getConfigInfosRequest(req, responseHolder)
        self.assertEquals(cm.exception.string,
                          "You need to be 'Manager' or 'MeetingManager' to get available categories for a user!")
        # now try with a 'pmManager'
        self.changeUser('pmManager')
        req._userToShowCategoriesFor = 'unexistingUserId'
        # the userToShowCategoriesFor must exists!
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).getConfigInfosRequest(req, responseHolder)
        self.assertEquals(cm.exception.string,
                          "Trying to get avaialble categories for an unexisting user 'unexistingUserId'!")
        # now get it.  The 'subproducts' category is only available to vendors (pmCreator2)
        req._userToShowCategoriesFor = 'pmCreator1'
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        response = SOAPView(self.portal, req).getConfigInfosRequest(req, responseHolder)
        resp = deserialize(response)
        # for 'pmCreator1', subproducts is not available
        self.assertFalse('<id>subproducts</id>' in resp)
        req._userToShowCategoriesFor = 'pmCreator2'
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        response = SOAPView(self.portal, req).getConfigInfosRequest(req, responseHolder)
        resp = deserialize(response)
        # but for 'pmCreator2', subproducts is available
        self.assertTrue('<id>subproducts</id>' in resp)

    def _getResultCategoriesForConfig(self, config):
        """
          Helper method for generating result displayed about categories of a MeetingConfig
        """
        # if not using categories, return empty categories list
        if not config.meta_type == 'MeetingConfig' or config.getUseGroupsAsCategories():
            return ''

        result = ""
        for cat in config.getCategories():
            result += """
    <categories xsi:type="ns1:BasicInfo">
      <UID>%s</UID>
      <id>%s</id>
      <title>%s</title>
      <description>%s</description>
    </categories>""" % (cat.UID(), cat.getId(), cat.Title(), cat.Description())
        # Empty description is represented like <description/>
        result = result.replace('<description></description>', '<description/>')
        return result

    def _getItemPositiveDecidedStatesFromConfig(self, config):
        """
          Helper method for generating result displayed
          about MeetingConfig.itemPositiveDecidedStates
        """
        result = ""
        for state in config.getItemPositiveDecidedStates():
            result += """
    <itemPositiveDecidedStates>%s</itemPositiveDecidedStates>""" \
        % (state)
        return result


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testSOAPGetConfigInfos, prefix='test_ws_'))
    return suite



##################################################
# file: WS4PM_server.py
#
# skeleton generated by "ZSI.generate.wsdl2dispatch.ServiceModuleWriter"
#      /srv/ZSI/bin/wsdl2py dumpedWSDL.txt -o . -b
#
##################################################

from ZSI.schema import GED, GTD
from ZSI.TCcompound import ComplexType, Struct
from WS4PM_types import *
from ZSI.ServiceContainer import ServiceSOAPBinding

# Messages  
testConnectionRequest = GED("http://ws4pm.imio.be", "testConnectionRequest").pyclass

testConnectionResponse = GED("http://ws4pm.imio.be", "testConnectionResponse").pyclass

checkIsLinkedRequest = GED("http://ws4pm.imio.be", "checkIsLinkedRequest").pyclass

checkIsLinkedResponse = GED("http://ws4pm.imio.be", "checkIsLinkedResponse").pyclass

getConfigInfosRequest = GED("http://ws4pm.imio.be", "getConfigInfosRequest").pyclass

getConfigInfosResponse = GED("http://ws4pm.imio.be", "getConfigInfosResponse").pyclass

getUserInfosRequest = GED("http://ws4pm.imio.be", "getUserInfosRequest").pyclass

getUserInfosResponse = GED("http://ws4pm.imio.be", "getUserInfosResponse").pyclass

getItemInfosRequest = GED("http://ws4pm.imio.be", "getItemInfosRequest").pyclass

getItemInfosResponse = GED("http://ws4pm.imio.be", "getItemInfosResponse").pyclass

getSingleItemInfosRequest = GED("http://ws4pm.imio.be", "getSingleItemInfosRequest").pyclass

getSingleItemInfosResponse = GED("http://ws4pm.imio.be", "getSingleItemInfosResponse").pyclass

getItemTemplateRequest = GED("http://ws4pm.imio.be", "getItemTemplateRequest").pyclass

getItemTemplateResponse = GED("http://ws4pm.imio.be", "getItemTemplateResponse").pyclass

searchItemsRequest = GED("http://ws4pm.imio.be", "searchItemsRequest").pyclass

searchItemsResponse = GED("http://ws4pm.imio.be", "searchItemsResponse").pyclass

meetingsAcceptingItemsRequest = GED("http://ws4pm.imio.be", "meetingsAcceptingItemsRequest").pyclass

meetingsAcceptingItemsResponse = GED("http://ws4pm.imio.be", "meetingsAcceptingItemsResponse").pyclass

createItemRequest = GED("http://ws4pm.imio.be", "createItemRequest").pyclass

createItemResponse = GED("http://ws4pm.imio.be", "createItemResponse").pyclass


# Service Skeletons
class WS4PM(ServiceSOAPBinding):
    soapAction = {}
    root = {}

    def __init__(self, post='', **kw):
        ServiceSOAPBinding.__init__(self, post)

    def soap_testConnection(self, ps, **kw):
        request = ps.Parse(testConnectionRequest.typecode)
        return request,testConnectionResponse()

    soapAction['http://ws4pm.imio.be/testConnection'] = 'soap_testConnection'
    root[(testConnectionRequest.typecode.nspname,testConnectionRequest.typecode.pname)] = 'soap_testConnection'

    def soap_checkIsLinked(self, ps, **kw):
        request = ps.Parse(checkIsLinkedRequest.typecode)
        return request,checkIsLinkedResponse()

    soapAction['http://ws4pm.imio.be/checkIsLinked'] = 'soap_checkIsLinked'
    root[(checkIsLinkedRequest.typecode.nspname,checkIsLinkedRequest.typecode.pname)] = 'soap_checkIsLinked'

    def soap_getConfigInfos(self, ps, **kw):
        request = ps.Parse(getConfigInfosRequest.typecode)
        return request,getConfigInfosResponse()

    soapAction['http://ws4pm.imio.be/getConfigInfos'] = 'soap_getConfigInfos'
    root[(getConfigInfosRequest.typecode.nspname,getConfigInfosRequest.typecode.pname)] = 'soap_getConfigInfos'

    def soap_getUserInfos(self, ps, **kw):
        request = ps.Parse(getUserInfosRequest.typecode)
        return request,getUserInfosResponse()

    soapAction['http://ws4pm.imio.be/getUserInfos'] = 'soap_getUserInfos'
    root[(getUserInfosRequest.typecode.nspname,getUserInfosRequest.typecode.pname)] = 'soap_getUserInfos'

    def soap_getItemInfos(self, ps, **kw):
        request = ps.Parse(getItemInfosRequest.typecode)
        return request,getItemInfosResponse()

    soapAction['http://ws4pm.imio.be/getItemInfos'] = 'soap_getItemInfos'
    root[(getItemInfosRequest.typecode.nspname,getItemInfosRequest.typecode.pname)] = 'soap_getItemInfos'

    def soap_getSingleItemInfos(self, ps, **kw):
        request = ps.Parse(getSingleItemInfosRequest.typecode)
        return request,getSingleItemInfosResponse()

    soapAction['http://ws4pm.imio.be/getSingleItemInfos'] = 'soap_getSingleItemInfos'
    root[(getSingleItemInfosRequest.typecode.nspname,getSingleItemInfosRequest.typecode.pname)] = 'soap_getSingleItemInfos'

    def soap_getItemTemplate(self, ps, **kw):
        request = ps.Parse(getItemTemplateRequest.typecode)
        return request,getItemTemplateResponse()

    soapAction['http://ws4pm.imio.be/getItemTemplate'] = 'soap_getItemTemplate'
    root[(getItemTemplateRequest.typecode.nspname,getItemTemplateRequest.typecode.pname)] = 'soap_getItemTemplate'

    def soap_searchItems(self, ps, **kw):
        request = ps.Parse(searchItemsRequest.typecode)
        return request,searchItemsResponse()

    soapAction['http://ws4pm.imio.be/searchItems'] = 'soap_searchItems'
    root[(searchItemsRequest.typecode.nspname,searchItemsRequest.typecode.pname)] = 'soap_searchItems'

    def soap_meetingsAcceptingItems(self, ps, **kw):
        request = ps.Parse(meetingsAcceptingItemsRequest.typecode)
        return request,meetingsAcceptingItemsResponse()

    soapAction['http://ws4pm.imio.be/meetingsAcceptingItems'] = 'soap_meetingsAcceptingItems'
    root[(meetingsAcceptingItemsRequest.typecode.nspname,meetingsAcceptingItemsRequest.typecode.pname)] = 'soap_meetingsAcceptingItems'

    def soap_createItem(self, ps, **kw):
        request = ps.Parse(createItemRequest.typecode)
        return request,createItemResponse()

    soapAction['http://ws4pm.imio.be/createItem'] = 'soap_createItem'
    root[(createItemRequest.typecode.nspname,createItemRequest.typecode.pname)] = 'soap_createItem'




# -*- coding: utf-8 -*-
#
# File: test_getuserinfos.py
#
# Copyright (c) 2013 by Imio.be
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

from imio.helpers.cache import cleanRamCacheFor
from imio.pm.ws.soap.soapview import SOAPView
from imio.pm.ws.tests.WS4PMTestCase import deserialize
from imio.pm.ws.tests.WS4PMTestCase import WS4PMTestCase
from imio.pm.ws.WS4PM_client import getUserInfosRequest
from imio.pm.ws.WS4PM_client import getUserInfosResponse

import ZSI


class testSOAPGetUserInfos(WS4PMTestCase):
    """
        Tests the soap.getUserInfosRequest method by accessing the real SOAP service
    """

    def test_ws_canNotGetUserInfosForAnotherUser(self):
        """
          Test that getting informations about another user is not possible
          except if the connected user is MeetingManager or Manager
        """
        # any PM user can have his own informations
        self.changeUser('pmCreator1')
        req = getUserInfosRequest()
        req._userId = 'pmCreator2'
        responseHolder = getUserInfosResponse()
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).getUserInfosRequest(req, responseHolder)
        self.assertEquals(cm.exception.string,
                          "You need to be 'Manager' or 'MeetingManager' to get "
                          "user informations for another user than 'pmCreator1'!")
        self.changeUser('pmManager')
        # pmManager can get informations for another user as it is a MeetingManager
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        response = SOAPView(self.portal, req).getUserInfosRequest(req, responseHolder)
        self.assertTrue(response._fullname == 'M. PMCreator Two')

    def test_ws_queriedUserMustExists(self):
        """
          Test that getting informations fails if the queried user does not exist
        """
        self.changeUser('pmManager')
        req = getUserInfosRequest()
        req._userId = 'unexistingUserId'
        responseHolder = getUserInfosResponse()
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).getUserInfosRequest(req, responseHolder)
        self.assertEquals(cm.exception.string,
                          "Trying to get user informations for an unexisting user 'unexistingUserId'!")

    def test_ws_getOwnUserInfosRequest(self):
        """
          Test that getting informations about a user returns valuable informations
        """
        # any PM user can have his own informations
        self.changeUser('pmCreator1')
        req = getUserInfosRequest()
        req._userId = 'pmCreator1'
        responseHolder = getUserInfosResponse()
        response = SOAPView(self.portal, req).getUserInfosRequest(req, responseHolder)
        self.assertTrue(response._fullname == 'M. PMCreator One')
        self.assertTrue(response._email == 'pmcreator1@plonemeeting.org')
        self.assertTrue(response._groups == [])
        # now as 'pmCreator2'
        self.changeUser('pmCreator2')
        req = getUserInfosRequest()
        req._userId = 'pmCreator2'
        responseHolder = getUserInfosResponse()
        response = SOAPView(self.portal, req).getUserInfosRequest(req, responseHolder)
        self.assertTrue(response._fullname == 'M. PMCreator Two')
        self.assertTrue(response._email == 'pmcreator2@plonemeeting.org')
        self.assertTrue(response._groups == [])

    def test_ws_getUserInfosShowGroups(self):
        """
          Test getting user informations including groups
        """
        # use pmManager that is member of 2 groups but only creator for one
        self.changeUser('pmManager')
        req = getUserInfosRequest()
        req._userId = 'pmManager'
        req._showGroups = True
        responseHolder = getUserInfosResponse()
        response = SOAPView(self.portal, req).getUserInfosRequest(req, responseHolder)
        resp = deserialize(response)
        expected = """<ns1:getUserInfosResponse xmlns:ns1="http://ws4pm.imio.be" """ \
                   """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
                   """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
                   """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
                   """xmlns:xsd="http://www.w3.org/2001/XMLSchema" """ \
                   """xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <fullname>M. PMManager</fullname>
  <email>pmmanager@plonemeeting.org</email>
  <groups xsi:type="ns1:BasicInfo">
    <UID>%s</UID>
    <id>developers</id>
    <title>Developers</title>
    <description/>
  </groups>
  <groups xsi:type="ns1:BasicInfo">
    <UID>%s</UID>
    <id>vendors</id>
    <title>Vendors</title>
    <description/>
  </groups>
</ns1:getUserInfosResponse>\n""" % (self.developers_uid, self.vendors_uid)
        self.assertEquals(expected, resp)
        # if we specify a particular suffix, only groups of this suffix are returned
        req._suffix = 'creators'
        responseHolder = getUserInfosResponse()
        response = SOAPView(self.portal, req).getUserInfosRequest(req, responseHolder)
        resp = deserialize(response)
        expected = """<ns1:getUserInfosResponse xmlns:ns1="http://ws4pm.imio.be" """ \
                   """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
                   """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
                   """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
                   """xmlns:xsd="http://www.w3.org/2001/XMLSchema" """ \
                   """xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <fullname>M. PMManager</fullname>
  <email>pmmanager@plonemeeting.org</email>
  <groups xsi:type="ns1:BasicInfo">
    <UID>%s</UID>
    <id>developers</id>
    <title>Developers</title>
    <description/>
  </groups>
</ns1:getUserInfosResponse>\n""" % (self.developers_uid, )


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testSOAPGetUserInfos, prefix='test_ws_'))
    return suite



# -*- coding: utf-8 -*-

from archetypes.schemaextender.field import ExtensionField
from archetypes.schemaextender.interfaces import IBrowserLayerAwareExtender
from archetypes.schemaextender.interfaces import ISchemaExtender
from imio.pm.ws.config import EXTERNAL_IDENTIFIER_FIELD_NAME
from imio.pm.ws.interfaces import IWS4PMLayer
from interfaces import IExternalIdentifierable
from Products.Archetypes.public import StringField
from Products.Archetypes.public import StringWidget
from zope.component import adapts
from zope.interface import implements


class ExternalIdentifierStringField(ExtensionField, StringField):
    """A string field that will contain an eventual external identifier."""


class ExternalIdentifierExtender(object):
    """ExternalIdentifier class"""

    implements(ISchemaExtender, IBrowserLayerAwareExtender)

    # adapts elements that provide the IExternalIdentifierable marker interface
    adapts(IExternalIdentifierable)

    layer = IWS4PMLayer

    fields = [
        ExternalIdentifierStringField(
            EXTERNAL_IDENTIFIER_FIELD_NAME,
            required=False,
            default='',
            searchable=False,
            languageIndependent=True,
            widget=StringWidget(
                label=(u"External identifier"),
                description=(u"You may enter an external identifier here if the "
                             u"item is created using an external service"),))
    ]

    def __init__(self, context):
        self.context = context

    def getOrder(self, schematas):
        """
        """
        return schematas

    def getFields(self):
        return self.fields



# -*- coding: utf-8 -*-
#
# File: test_getiteminfos.py
#
# GNU General Public License (GPL)
#

from datetime import datetime
from imio.pm.ws.soap.soapview import SOAPView
from imio.pm.ws.tests.WS4PMTestCase import WS4PMTestCase
from imio.pm.ws.WS4PM_client import meetingsAcceptingItemsRequest
from imio.pm.ws.WS4PM_client import meetingsAcceptingItemsResponse

import ZSI


class testSOAPMeetingsAcceptingItems(WS4PMTestCase):
    """
        Tests the soap.meetingsAcceptingItems method by accessing the real SOAP service
    """

    def test_ws_meetingAcceptingItems(self):
        """
          Test that getting meetings accepting items works
        """
        # create 2 meetings and test
        # we are not testing the MeetingConfig.getMeetingsAcceptingItems method
        # but we are testing that using the WS works...
        self.changeUser('pmManager')
        # by default, no Meeting exists...
        self.failUnless(len(self.portal.portal_catalog(portal_type='MeetingPga')) == 0)
        meeting1 = self.create('Meeting', datetime(2015, 1, 1))
        meeting2 = self.create('Meeting', datetime(2015, 2, 2))
        req = meetingsAcceptingItemsRequest()
        responseHolder = meetingsAcceptingItemsResponse()
        # a known MeetingConfig id is required
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).meetingsAcceptingItemsRequest(req, responseHolder)
        self.assertEquals(cm.exception.string,
                          "Unknown meetingConfigId : 'None'!")
        req._meetingConfigId = self.meetingConfig.getId()
        response = SOAPView(self.portal, req).meetingsAcceptingItemsRequest(req, responseHolder)
        # returned meetings are sorted by date ascending
        self.assertTrue(len(response._meetingInfo) == 2)
        self.assertTrue(response._meetingInfo[0]._UID == meeting1.UID())
        self.assertTrue(response._meetingInfo[0]._date == meeting1.date.utctimetuple())
        self.assertTrue(response._meetingInfo[1]._UID == meeting2.UID())
        self.assertTrue(response._meetingInfo[1]._date == meeting2.date.utctimetuple())


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testSOAPMeetingsAcceptingItems, prefix='test_ws_'))
    return suite



# -*- coding: utf-8 -*-

from imio.pm.ws.soap.soapview import SOAPView
from imio.pm.ws.testing import WS4PM_PM_TEST_PROFILE_FUNCTIONAL
from imio.pm.ws.WS4PM_client import createItemRequest
from imio.pm.ws.WS4PM_client import createItemResponse
from imio.pm.ws.WS4PM_client import getItemInfosRequest
from imio.pm.ws.WS4PM_client import getItemInfosResponse
from imio.pm.ws.WS4PM_client import getSingleItemInfosRequest
from imio.pm.ws.WS4PM_client import getSingleItemInfosResponse
from imio.pm.ws.WS4PM_client import searchItemsResponse
from lxml import etree
from Products.PloneMeeting.tests.PloneMeetingTestCase import PloneMeetingTestCase
from ZSI import SoapWriter
from ZSI import TC
from ZSI.schema import GTD

import os


class WS4PMTestCase(PloneMeetingTestCase):
    '''Base class for defining WS4PM test cases.'''

    layer = WS4PM_PM_TEST_PROFILE_FUNCTIONAL

    def setUp(self):
        """ """
        PloneMeetingTestCase.setUp(self)
        # remove items of the config
        self.changeUser('admin')
        self._removeConfigObjectsFor(self.meetingConfig)
        self._removeConfigObjectsFor(self.meetingConfig2)
        # make sure 'detailedDescription' optional field is selected for both configs
        itemAttrs = self.meetingConfig.getUsedItemAttributes()
        self.meetingConfig.setUsedItemAttributes(itemAttrs + ('detailedDescription', ))
        itemAttrs = self.meetingConfig2.getUsedItemAttributes()
        self.meetingConfig2.setUsedItemAttributes(itemAttrs + ('detailedDescription', ))
        # use the 'plonegov-assembly' MeetingConfig that use real categories,
        # not useGroupsAsCategories, even if it could be changed during a test
        self.meetingConfig = self.meetingConfig2
        self.usedMeetingConfigId = 'plonegov-assembly'

    def _prepareCreationData(self):
        """
          Helper method for creating an item using the SOAP method createItem
        """
        # make sure we enabled the 'detailedDescription' optional field
        itemAttrs = self.meetingConfig.getUsedItemAttributes()
        self.meetingConfig.setUsedItemAttributes(itemAttrs + ('detailedDescription', ))
        req = createItemRequest()
        req._meetingConfigId = self.usedMeetingConfigId
        req._proposingGroupId = 'developers'
        CreationData = GTD('http://ws4pm.imio.be', 'CreationData')('').pyclass()
        CreationData._title = 'My new item title'
        CreationData._category = 'development'
        CreationData._description = '<p>Description</p>'
        CreationData._detailedDescription = '<p>Detailed description</p>'
        CreationData._decision = '<p>Décision</p>'
        req._creationData = CreationData
        return req

    def _prepareAnnexInfo(self, **data):
        """
          Helper method for adding an annex to a created item
        """
        # create one annex
        AnnexInfo = GTD('http://ws4pm.imio.be', 'AnnexInfo')('').pyclass()
        AnnexInfo._title = data.get('title')
        # optional
        AnnexInfo._annexTypeId = data.get('annexTypeId', '')
        # optional
        AnnexInfo._filename = data.get('filename', '')
        if data.get('file'):
            annex_file = open(os.path.join(os.path.dirname(__file__), data.get('file')))
            AnnexInfo._file = annex_file.read()
            annex_file.close()
        return AnnexInfo

    def _createItem(self, req):
        """
          Create the item with data given in req parameter
        """
        responseHolder = createItemResponse()
        response = SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        newItem = self.portal.uid_catalog(UID=response._UID)[0].getObject()
        return newItem, response

    def _getItemInfos(self,
                      itemUID,
                      showAnnexes=False,
                      allowed_annexes_types=[],
                      include_annex_binary=True,
                      showAssembly=False,
                      showExtraInfos=False,
                      showTemplates=False,
                      showEmptyValues=True,
                      toBeDeserialized=True,
                      useSingleItemInfos=False):
        """
          Call getItemInfos SOAP method with given itemUID parameter
        """
        if useSingleItemInfos:
            req = getSingleItemInfosRequest()
        else:
            req = getItemInfosRequest()
        req._UID = itemUID
        if showAnnexes:
            req._showAnnexes = True
        if allowed_annexes_types:
            req._allowed_annexes_types = allowed_annexes_types
        if not include_annex_binary:
            req._include_annex_binary = False
        if showExtraInfos:
            req._showExtraInfos = True
        if showAssembly:
            req._showAssembly = True
        if showTemplates:
            req._showTemplates = True
        req._showEmptyValues = showEmptyValues

        if useSingleItemInfos:
            responseHolder = getSingleItemInfosResponse()
            response = SOAPView(self.portal, req).getSingleItemInfosRequest(req, responseHolder)
        else:
            responseHolder = getItemInfosResponse()
            response = SOAPView(self.portal, req).getItemInfosRequest(req, responseHolder)

        if toBeDeserialized:
            return deserialize(response)
        else:
            return response

    def _searchItems(self, req):
        """
          Search items with data of req parameter
        """
        responseHolder = searchItemsResponse()
        response = SOAPView(self.portal, req).searchItemsRequest(req, responseHolder)
        return deserialize(response)

    def _check_after_inTheNameOf(self):
        """
          After a call using inTheNameOf, check that _listAllowedRolesAndUsers
          is still correct. This check a bug that was making
          ToolPloneMeeting.get_plone_groups_for_user caching wrong.
          This happened before because ToolPloneMeeting.get_plone_groups_for_user
          had a different value between request.AUTHENTICATED_USER and
          api.user.get_current
        """
        for user_id in ('pmCreator1', 'pmCreator2', 'pmManager'):
            self.changeUser(user_id)
            # 2 extra values, the user_id and role "Anonymous"
            self.assertEqual(
                len(self.catalog._listAllowedRolesAndUsers(self.member)),
                len(self.member.getGroups()) + len(self.member.getRoles()) + 2)


def serializeRequest(request):
    tc = getattr(request, 'typecode', None)
    sw = SoapWriter(nsdict={}, header=True, outputclass=None,
                    encodingStyle=None)
    return str(sw.serialize(request, tc))


def deserialize(objectToDeserialize):
    sw = SoapWriter(nsdict={}, header=True, outputclass=None,
                    encodingStyle=None)
    tc = TC.Any(pname=None, aslist=False)
    deserializedObject = sw.serialize(objectToDeserialize, tc).body
    root = etree.XML(str(deserializedObject))
    body = root[0]
    return etree.tostring(body, encoding='utf-8', pretty_print=True)



# -*- coding: utf-8 -*-
#
# File: test_createitem.py
#
# GNU General Public License (GPL)
#

from imio.helpers.cache import cleanRamCacheFor
from imio.pm.ws.soap.soapview import MIMETYPE_NOT_FOUND_OF_ANNEX_WARNING
from imio.pm.ws.soap.soapview import MULTIPLE_EXTENSION_FOR_MIMETYPE_OF_ANNEX_WARNING
from imio.pm.ws.soap.soapview import SOAPView
from imio.pm.ws.soap.soapview import WRONG_HTML_WARNING
from imio.pm.ws.tests.WS4PMTestCase import deserialize
from imio.pm.ws.tests.WS4PMTestCase import serializeRequest
from imio.pm.ws.tests.WS4PMTestCase import WS4PMTestCase
from imio.pm.ws.WS4PM_client import createItemResponse
from magic import MagicException
from Products.PloneMeeting.config import ITEM_NO_PREFERRED_MEETING_VALUE
from Products.PloneMeeting.utils import get_annexes
from zope.i18n import translate
from ZSI.schema import GTD

import base64
import magic
import unittest
import ZSI


class testSOAPCreateItem(WS4PMTestCase):
    """
        Tests the soap.createItemRequest method by accessing the real SOAP service
    """

    def test_ws_createItemRequest(self):
        """
          In the default test configuration, the user 'pmCreator1' can create an item for
          proposingGroup 'developers' in the MeetingConfig 'plonegov-assembly'
        """
        # by default no item exists
        self.changeUser('pmCreator1')
        self.failUnless(len(self.portal.portal_catalog(portal_type='MeetingItemPga')) == 0)
        req = self._prepareCreationData()
        # This is what the sent enveloppe should looks like, note that the decision is "Décision<strong>wrongTagd</p>"
        # instead of '<p>Décision</p>' so we check accents and missing <p></p>
        req._creationData._decision = 'Décision<strong>wrongTagd</p>'
        # Serialize the request so it can be easily tested
        request = serializeRequest(req)
        expected = """POST /plone/createItemRequest HTTP/1.0
Authorization: Basic %s:%s
Content-Length: 102
Content-Type: text/xml
SOAPAction: /
<SOAP-ENV:Envelope xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
"""xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
"""xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">""" \
"""<SOAP-ENV:Header></SOAP-ENV:Header><SOAP-ENV:Body xmlns:ns1="http://ws4pm.imio.be"><ns1:createItemRequest>""" \
"""<meetingConfigId>plonegov-assembly</meetingConfigId><proposingGroupId>developers</proposingGroupId>""" \
"""<creationData xsi:type="ns1:CreationData"><title>My new item title</title><category>development</category>""" \
"""<description>&lt;p&gt;Description&lt;/p&gt;</description>""" \
"""<detailedDescription>&lt;p&gt;Detailed description&lt;/p&gt;</detailedDescription>""" \
"""<decision>D\xc3\xa9cision&lt;strong&gt;wrongTagd&lt;/p&gt;</decision></creationData><cleanHtml>true</cleanHtml>""" \
"""</ns1:createItemRequest>""" \
"""</SOAP-ENV:Body></SOAP-ENV:Envelope>""" % ('pmCreator1', 'meeting')
        result = """POST /plone/createItemRequest HTTP/1.0
Authorization: Basic %s:%s
Content-Length: 102
Content-Type: text/xml
SOAPAction: /
%s""" % ('pmCreator1', 'meeting', request)
        self.assertEqual(expected, result)
        # now really use the SOAP method to create an item
        newItem, response = self._createItem(req)
        newItemUID = newItem.UID()
        resp = deserialize(response)
        expected = """<ns1:createItemResponse xmlns:ns1="http://ws4pm.imio.be" """ \
            """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """\
            """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
            """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
            """xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <UID>%s</UID>
  <warnings>%s</warnings>
</ns1:createItemResponse>
""" % (newItemUID, translate(WRONG_HTML_WARNING,
                             domain='imio.pm.ws',
                             mapping={'item_path': newItem.absolute_url_path(),
                                      'creator': 'pmCreator1'},
                             context=self.request)
       )
        self.assertEqual(expected, resp)
        # the item is actually created
        self.failUnless(len(self.portal.portal_catalog(portal_type='MeetingItemPga', UID=newItemUID)) == 1)
        # responseHolder for tests here above
        responseHolder = createItemResponse()
        # check that we can create an item with a NoneType HTML field
        req._creationData._decision = None
        newItemWithEmptyDecisionUID = SOAPView(self.portal, req).createItemRequest(req, responseHolder)._UID
        self.failUnless(len(self.portal.portal_catalog(portal_type='MeetingItemPga',
                                                       UID=newItemWithEmptyDecisionUID)) == 1)
        # No matter how the item is created, with or without a decision, every HTML fields are surrounded by <p></p>
        obj = self.portal.portal_catalog(portal_type='MeetingItemPga', UID=newItemWithEmptyDecisionUID)[0].getObject()
        self.failIf(obj.getDecision(keepWithNext=False) != '<p></p>')

    def test_ws_createItemRaisedZSIFaults(self):
        """
          Test SOAP service behaviour when creating an item with some wrong arguments
        """
        # by default no item exists
        self.changeUser('pmCreator1')
        req = self._prepareCreationData()
        responseHolder = createItemResponse()
        # the title is mandatory
        req._creationData._title = None
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(cm.exception.string, "A 'title' is mandatory!")
        req._creationData._title = 'A valid title'
        # the meetingConfigId must exists
        req._meetingConfigId = 'wrongMeetingConfigId'
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(cm.exception.string, "Unknown meetingConfigId : 'wrongMeetingConfigId'!")
        req._meetingConfigId = self.usedMeetingConfigId
        # the connected user must be able to create an item for the given proposingGroupId
        req._proposingGroupId = 'vendors'
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(cm.exception.string, "'pmCreator1' can not create items for the 'vendors' group!")
        # the connected user must be able to create an item with the given category
        # set back correct proposingGroup
        req._proposingGroupId = 'developers'
        # if category is mandatory and empty, it raises ZSI.Fault
        self.meetingConfig.setUseGroupsAsCategories(False)
        req._creationData._category = None
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(cm.exception.string, "In this config, category is mandatory!")
        # wrong category and useGroupsAsCategories, ZSI.Fault
        self.meetingConfig.setUseGroupsAsCategories(True)
        req._creationData._category = 'wrong-category-id'
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(cm.exception.string,
                         "This config does not use categories, the given 'wrong-category-id' "
                         "category can not be used!")
        # wrong category and actually accepting categories, aka useGroupsAsCategories to False
        self.meetingConfig.setUseGroupsAsCategories(False)
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(cm.exception.string,
                         "'wrong-category-id' is not available for the 'developers' group!")
        # if the user trying to create an item has no member area, a ZSI.Fault is raised
        # remove the 'pmCreator2' personal area
        self.changeUser('admin')
        self.portal.Members.manage_delObjects(ids=['pmCreator2'])
        req._proposingGroupId = 'vendors'
        self.changeUser('pmCreator2')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(cm.exception.string, "No member area for 'pmCreator2'.  Never connected to PloneMeeting?")

    def test_ws_createItemWithOptionalFields(self):
        """
          Test SOAP service behaviour when creating an item with some optional fields
        """
        # by default no item exists
        self.changeUser('pmCreator1')
        req = self._prepareCreationData()
        responseHolder = createItemResponse()
        # we can not use an optional field that is not activated in the current MeetingConfig
        self.assertTrue('motivation' not in self.meetingConfig.getUsedItemAttributes())
        req._creationData._motivation = '<p>Motivation sample text</p>'
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(cm.exception.string,
                         "The optional field \"motivation\" is not activated in this configuration!")
        # if we activate it, then the resulting item is correct
        self.meetingConfig.setUsedItemAttributes(self.meetingConfig.getUsedItemAttributes() + ('motivation', ))
        newItem, response = self._createItem(req)
        self.assertTrue(newItem.getMotivation() == '<p>Motivation sample text</p>')

    def test_ws_createItemToDiscuss(self):
        """
          Test SOAP service behaviour when creating an item using toDiscuss :
          - optional field so only useable when relevant;
          - if not set, then default value is used.
        """
        cfg = self.meetingConfig
        # by default no item exists
        self.changeUser('pmCreator1')
        req = self._prepareCreationData()
        responseHolder = createItemResponse()
        # we can not use an optional field that is not activated in the current MeetingConfig
        cfg.setUsedItemAttributes(('description', 'detailedDescription',))
        self.assertFalse('toDiscuss' in cfg.getUsedItemAttributes())
        # set toDiscuss to True
        req._creationData._toDiscuss = True
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(cm.exception.string,
                         "The optional field \"toDiscuss\" is not activated in this configuration!")
        # if we activate it, then the resulting item is correct
        cfg.setUsedItemAttributes(cfg.getUsedItemAttributes() + ('toDiscuss', ))

        # create item first time when default is True, then False
        # as given in soap request, it is True each time
        cfg.setToDiscussDefault(False)
        newItem, response = self._createItem(req)
        self.assertTrue(newItem.getToDiscuss())
        cfg.setToDiscussDefault(True)
        newItem, response = self._createItem(req)
        self.assertTrue(newItem.getToDiscuss())
        # if not set in soap request, parameter is ignored, again with default True and False
        # as not given in soap request, it is the default defined value
        req._creationData._toDiscuss = None
        cfg.setToDiscussDefault(False)
        newItem, response = self._createItem(req)
        self.assertFalse(newItem.getToDiscuss())
        cfg.setToDiscussDefault(True)
        newItem, response = self._createItem(req)
        self.assertTrue(newItem.getToDiscuss())

    def test_ws_createItemWithOneAnnexRequest(self):
        """
          Test SOAP service behaviour when creating items with one annex
        """
        # by default no item exists
        self.changeUser('pmCreator1')
        req = self._prepareCreationData()
        data = {'title': 'My annex 1', 'filename': 'smallTestFile.pdf', 'file': 'smallTestFile.pdf'}
        req._creationData._annexes = [self._prepareAnnexInfo(**data)]
        annex = req._creationData._annexes[0]
        # Serialize the request so it can be easily tested
        request = serializeRequest(req)
        # This is what the sent enveloppe should looks like
        expected = """POST /plone/createItemRequest HTTP/1.0
Authorization: Basic %s:%s
Content-Length: 102
Content-Type: text/xml
SOAPAction: /
<SOAP-ENV:Envelope xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
"""xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
"""xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">""" \
"""<SOAP-ENV:Header></SOAP-ENV:Header><SOAP-ENV:Body xmlns:ns1="http://ws4pm.imio.be">""" \
"""<ns1:createItemRequest><meetingConfigId>plonegov-assembly</meetingConfigId>""" \
"""<proposingGroupId>developers</proposingGroupId><creationData xsi:type="ns1:CreationData">""" \
"""<title>My new item title</title><category>development</category>""" \
"""<description>&lt;p&gt;Description&lt;/p&gt;</description>""" \
"""<detailedDescription>&lt;p&gt;Detailed description&lt;/p&gt;</detailedDescription>""" \
"""<decision>&lt;p&gt;Décision&lt;/p&gt;</decision>""" \
"""<annexes xsi:type="ns1:AnnexInfo"><title>%s</title><annexTypeId>%s</annexTypeId><filename>%s</filename><file>
%s</file></annexes></creationData><cleanHtml>true</cleanHtml></ns1:createItemRequest>""" \
"""</SOAP-ENV:Body></SOAP-ENV:Envelope>""" % ('pmCreator1', 'meeting', annex._title, annex._annexTypeId,
                                              annex._filename, base64.encodestring(annex._file))
        result = """POST /plone/createItemRequest HTTP/1.0
Authorization: Basic %s:%s
Content-Length: 102
Content-Type: text/xml
SOAPAction: /
%s""" % ('pmCreator1', 'meeting', request)
        self.assertEqual(expected, result)
        newItem, response = self._createItem(req)
        # now check the created item have the annex
        annexes = get_annexes(newItem)
        # the annex is actually created
        self.failUnless(len(annexes) == 1)
        # the annex mimetype is correct
        annex = annexes[0]
        self.failUnless(annex.file.contentType == 'application/pdf')
        # the annex metadata are ok
        self.assertEqual(annex.Title(), 'My annex 1')
        self.assertEqual(annex.content_category,
                         'plonegov-assembly-annexes_types_-_item_annexes_-_financial-analysis')

    @unittest.skip("This test is skipped, remove decorator if you use libmagic/file < 5.10")
    def test_ws_createItemWithAnnexNotRecognizedByLibmagicRequest(self):
        """
          Test SOAP service behaviour when creating items with one annex that is not recognized
          by libmagic.  The annex used here is a MS Word .doc file that fails to be recognized when
          using libmagic/file 5.09
        """
        self.changeUser('pmCreator1')
        req = self._prepareCreationData()
        data = {'title': 'My crashing created annex',
                'filename': 'file_crash_libmagic.doc',
                'file': 'file_crash_libmagic.doc'}
        req._creationData._annexes = [self._prepareAnnexInfo(**data)]
        annex = req._creationData._annexes[0]
        # first make sure this file crash libmagic
        self.assertRaises(MagicException, magic.Magic(mime=True).from_buffer, annex._file)
        newItem, response = self._createItem(req)
        # the annex is nevertheless created and correctly recognized because it had a correct file extension
        annexes = get_annexes(newItem)
        self.failUnless(len(annexes) == 1)
        # the annex mimetype is correct
        annex = annexes[0]
        self.failUnless(annex.file.contentType == 'application/msword')
        # the annex metadata are ok
        self.failUnless(annex.Title() == 'My crashing created annex' and
                        annex.getMeetingFileType().getId() == 'financial-analysis')
        # if libmagic crash and no valid filename provided, the annex is not created
        data = {'title': 'My crashing NOT created annex',
                'filename': 'file_crash_libmagic_without_extension',
                'file': 'file_crash_libmagic.doc'}
        req._creationData._annexes = [self._prepareAnnexInfo(**data)]
        newItem2, response = self._createItem(req)
        self.failUnless(len(get_annexes(newItem2)) == 0)
        # a warning specifying that annex was not added because mimetype could
        # not reliabily be found is added in the response
        self.assertEqual(response._warnings, [translate(MIMETYPE_NOT_FOUND_OF_ANNEX_WARNING,
                                                        domain='imio.pm.ws',
                                                        mapping={'annex_path': (data['filename']),
                                                                 'item_path': newItem2.absolute_url_path()},
                                                        context=self.portal.REQUEST)])

    def test_ws_createItemWithSeveralAnnexesRequest(self):
        """
          Test SOAP service behaviour when creating items with several annexes of different types
        """
        # by default no item exists
        self.changeUser('pmCreator1')
        req = self._prepareCreationData()
        # add 4 extra annexes
        # no data give, some default values are used (smallTestFile.pdf)
        data1 = {'title': 'My annex 1',
                 'filename': 'smallTestFile.pdf',
                 'file': 'smallTestFile.pdf'}
        # other annexTypeId than the default one
        data2 = {'title': 'My annex 2',
                 'filename': 'arbitraryFilename.odt',
                 'file': 'mediumTestFile.odt',
                 'annexTypeId': 'budget-analysis'}
        # a wrong annexTypeId and a test with a large msword document
        data3 = {'title': 'My annex 3',
                 'filename': 'largeTestFile.doc',
                 'file': 'largeTestFile.doc',
                 'annexTypeId': 'wrong-annexTypeId'}
        # empty file data provided, at the end, the annex is not created but the item is correctly created
        data4 = {'title': 'My annex 4',
                 'filename': 'emptyTestFile.txt',
                 'file': 'emptyTestFile.txt',
                 'annexTypeId': 'budget-analysis'}
        # a file that will have several extensions found in mimetypes_registry
        # is not handled if no valid filename is provided
        data5 = {'title': 'My annex 5',
                 'filename': 'notValidFileNameNoExtension',
                 'file': 'octetStreamTestFile.bin',
                 'annexTypeId': 'budget-analysis'}
        # but if the filename is valid, then the annex is handled
        data6 = {'title': 'My annex 6',
                 'filename': 'validExtension.bin',
                 'file': 'octetStreamTestFile.bin',
                 'annexTypeId': 'budget-analysis'}
        req._creationData._annexes = [self._prepareAnnexInfo(**data1), self._prepareAnnexInfo(**data2),
                                      self._prepareAnnexInfo(**data3), self._prepareAnnexInfo(**data4),
                                      self._prepareAnnexInfo(**data5), self._prepareAnnexInfo(**data6)]
        # serialize the request so it can be easily tested
        request = serializeRequest(req)
        # build annexes part of the envelope
        annexesEnveloppePart = ""
        for annex in req._creationData._annexes:
            annexesEnveloppePart = annexesEnveloppePart + """<annexes xsi:type="ns1:AnnexInfo"><title>%s</title>""" \
                """<annexTypeId>%s</annexTypeId><filename>%s</filename><file>
%s</file></annexes>""" % (annex._title, annex._annexTypeId, annex._filename, base64.encodestring(annex._file))
        # This is what the sent enveloppe should looks like
        expected = """POST /plone/createItemRequest HTTP/1.0
Authorization: Basic %s:%s
Content-Length: 102
Content-Type: text/xml
SOAPAction: /
<SOAP-ENV:Envelope xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
"""xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
"""xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">""" \
"""<SOAP-ENV:Header></SOAP-ENV:Header><SOAP-ENV:Body xmlns:ns1="http://ws4pm.imio.be"><ns1:createItemRequest>""" \
"""<meetingConfigId>plonegov-assembly</meetingConfigId><proposingGroupId>developers</proposingGroupId>""" \
"""<creationData xsi:type="ns1:CreationData"><title>My new item title</title><category>development</category>""" \
"""<description>&lt;p&gt;Description&lt;/p&gt;</description>""" \
"""<detailedDescription>&lt;p&gt;Detailed description&lt;/p&gt;</detailedDescription>""" \
"""<decision>&lt;p&gt;Décision&lt;/p&gt;</decision>""" \
"""%s</creationData><cleanHtml>true</cleanHtml></ns1:createItemRequest></SOAP-ENV:Body></SOAP-ENV:Envelope>""" \
% ('pmCreator1', 'meeting', annexesEnveloppePart)
        result = """POST /plone/createItemRequest HTTP/1.0
Authorization: Basic %s:%s
Content-Length: 102
Content-Type: text/xml
SOAPAction: /
%s""" % ('pmCreator1', 'meeting', request)
        self.assertEqual(expected, result)
        newItem, response = self._createItem(req)
        annexes = get_annexes(newItem)
        # 4 annexes are actually created
        self.failUnless(len(annexes) == 4)
        # the annexes mimetype are corrects
        self.failUnless(annexes[0].file.contentType == 'application/pdf')
        self.failUnless(annexes[1].file.contentType == 'application/vnd.oasis.opendocument.text')
        self.failUnless(annexes[2].file.contentType == 'application/msword')
        self.failUnless(annexes[3].file.contentType == 'application/octet-stream')
        # the annexes metadata are ok
        self.failUnless(
            annexes[0].Title() == 'My annex 1' and
            annexes[0].content_category == 'plonegov-assembly-annexes_types_-_item_annexes_-_financial-analysis')
        self.failUnless(
            annexes[1].Title() == 'My annex 2' and
            annexes[1].content_category == 'plonegov-assembly-annexes_types_-_item_annexes_-_budget-analysis')
        # meetingFileType is back to default one when a wrong file type is given in the annexInfo
        self.failUnless(
            annexes[2].Title() == 'My annex 3' and
            annexes[2].content_category == 'plonegov-assembly-annexes_types_-_item_annexes_-_financial-analysis')
        self.failUnless(
            annexes[3].Title() == 'My annex 6' and
            annexes[3].content_category == 'plonegov-assembly-annexes_types_-_item_annexes_-_budget-analysis')
        # annexes filename are the ones defined in the 'filename', either it is generated
        self.failUnless(annexes[0].file.filename == u'smallTestFile.pdf')
        self.failUnless(annexes[1].file.filename == u'arbitraryFilename.odt')
        self.failUnless(annexes[2].file.filename == u'largeTestFile.doc')
        self.failUnless(annexes[3].file.filename == u'validExtension.bin')
        # now try to create an item with an annex that has no file
        # when file attribute is not provided, the annex is not created
        data = {'title': 'My annex 7',
                'filename': 'validExtension.bin',
                'annexTypeId': 'budget-analysis'}
        req._creationData._annexes = [self._prepareAnnexInfo(**data), ]
        self.assertEqual(req._creationData._annexes[0]._file, None)
        newItem, response = self._createItem(req)
        annexes = get_annexes(newItem)
        # no annexes have been added
        self.assertEqual(len(annexes), 0)

    def test_ws_createItemWithWarnings(self):
        """
          Test that during item creation, if non blocker errors occur (warnings), it is displayed in the response
        """
        # if the proposed HTML of one of the rich field is wrong
        # it is reworked by BeautifulSoup and a warning is displayed
        self.changeUser('pmCreator1')
        req = self._prepareCreationData()
        wrongHTML = '<p>Wrong HTML<strong></p></strong>'
        req._creationData._decision = wrongHTML
        newItem, response = self._createItem(req)
        resp = deserialize(response)
        expected = """<ns1:createItemResponse xmlns:ns1="http://ws4pm.imio.be" """ \
            """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
            """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
            """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
            """xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <UID>%s</UID>
  <warnings>%s</warnings>
</ns1:createItemResponse>
""" % (newItem.UID(), translate(WRONG_HTML_WARNING,
                                domain='imio.pm.ws',
                                mapping={'item_path': newItem.absolute_url_path(),
                                         'creator': 'pmCreator1'},
                                context=self.request))
        self.assertEqual(expected, resp)
        # now test warnings around file mimetype
        data = {'title': 'My annex with spécial char and no filename',
                'filename': '',
                'file': 'octetStreamTestFile.bin',
                'annexTypeId': 'budget-analysis'}
        req._creationData._annexes = [self._prepareAnnexInfo(**data)]
        # several extensions found and no valid filename extension, the annex is not created and a warning is added
        newItem, response = self._createItem(req)
        resp = deserialize(response)
        # 2 warnings are returned
        expected = """<ns1:createItemResponse xmlns:ns1="http://ws4pm.imio.be" """ \
            """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
            """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
            """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
            """xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <UID>%s</UID>
  <warnings>%s</warnings>
  <warnings>%s</warnings>
</ns1:createItemResponse>
""" % (newItem.UID(), translate(
                WRONG_HTML_WARNING,
                domain='imio.pm.ws',
                mapping={'item_path': newItem.absolute_url_path(),
                         'creator': 'pmCreator1'},
                context=self.request),
                translate(
                    MULTIPLE_EXTENSION_FOR_MIMETYPE_OF_ANNEX_WARNING,
                    domain='imio.pm.ws',
                    mapping={'mime': 'application/octet-stream',
                             'annex_path': unicode(data['title'], 'utf-8'),
                             'item_path': newItem.absolute_url_path()},
                    context=self.request))
        expected = expected.encode('utf-8')
        self.assertEqual(expected, resp)

    def test_ws_createItemInTheNameOf(self):
        """
          It is possible for Managers and MeetingManagers to create an item inTheNameOf another user
          Every other checks are made except that for using the inTheNameOf functionnality :
          - the calling user must be 'Manager' or 'MeetingManager'
          - the created item is finally like if created by the inTheNameOf user
        """
        self.meetingConfig.setUseGroupsAsCategories(False)
        # check first a working example the degrades it...
        # and every related informations (creator, ownership, ...) are corretly linked to inTheNameOf user
        self.changeUser('pmManager')
        req = self._prepareCreationData()
        req._inTheNameOf = 'pmCreator2'
        req._proposingGroupId = 'vendors'
        req._creationData._category = 'development'
        data = {'title': 'My annex 1', 'filename': 'smallTestFile.pdf', 'file': 'smallTestFile.pdf'}
        req._creationData._annexes = [self._prepareAnnexInfo(**data)]
        responseHolder = createItemResponse()
        response = SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        # as we switch user while using inTheNameOf, make sure we have
        # falled back to original user
        self.assertTrue(self.portal.portal_membership.getAuthenticatedMember().getId() == 'pmManager')
        # make also sure that cached methods using user_id are correct as well
        newItem = self.portal.uid_catalog(UID=response._UID)[0].getObject()
        # as the item is really created by the inTheNameOf user, everything is correct
        self.assertEqual(newItem.Creator(), 'pmCreator2')
        self.assertEqual(newItem.owner_info()['id'], 'pmCreator2')
        # with those data but with a non 'Manager'/'MeetingManager', it fails
        self.changeUser('pmCreator1')
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(cm.exception.string,
                         "You need to be 'Manager' or 'MeetingManager' to create an item 'inTheNameOf'!")
        # now use the MeetingManager but specify a proposingGroup the inTheNameOf user can not create for
        self.changeUser('pmManager')
        req._proposingGroupId = 'developers'
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(cm.exception.string, "'pmCreator2' can not create items for the 'developers' group!")
        # now for an unexisting inTheNameOf userid
        req._inTheNameOf = 'unexistingUserId'
        # set back correct proposingGroupId
        req._proposingGroupId = 'vendors'
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(cm.exception.string,
                         "Trying to create an item 'inTheNameOf' an unexisting user 'unexistingUserId'!")
        # create an itemInTheNameOf a user having no personal area...
        # if the user trying to create an item has no member area, a ZSI.Fault is raised
        # remove the 'pmCreator2' personal area
        self.changeUser('admin')
        # remove the created item because we can not remove a folder containing items
        # it would raise a BeforeDeleteException in PloneMeeting
        newItem.aq_inner.aq_parent.manage_delObjects(ids=[newItem.getId(), ])
        self.portal.Members.manage_delObjects(ids=['pmCreator2'])
        self.changeUser('pmManager')
        req._inTheNameOf = 'pmCreator2'
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(cm.exception.string, "No member area for 'pmCreator2'.  Never connected to PloneMeeting?")
        # test that _listAllowedRolesAndUsers is not messed up
        # this happened before because ToolPloneMeeting.get_plone_groups_for_user
        # had a different value between request.AUTHENTICATED_USER and api.user.get_current
        self._check_after_inTheNameOf()

    def test_ws_createItemWithPreferredMeeting(self):
        """
          It is possible to specify a preferred meeting, but the given
          preferred meeting UID must be a meeting accepting items.
        """
        self.changeUser('pmManager')
        # create a fresh meeting that will accept items
        meeting = self.create('Meeting')
        req = self._prepareCreationData()
        req._creationData._preferredMeeting = 'unexisting_meeting_UID'
        responseHolder = createItemResponse()
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(cm.exception.string,
                         "The given preferred meeting UID (unexisting_meeting_UID) is not a meeting accepting items!")
        req._creationData._preferredMeeting = meeting.UID()
        response = SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        # an item has been created with correct preferredMeeting
        item = self.portal.portal_catalog(UID=response._UID)[0].getObject()
        self.assertTrue(item.getPreferredMeeting() == meeting.UID())
        # if no preferredMeeting is provided, the default value 'whatever' is used
        req._creationData._preferredMeeting = None
        response = SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        item = self.portal.portal_catalog(UID=response._UID)[0].getObject()
        self.assertTrue(item.getPreferredMeeting() == ITEM_NO_PREFERRED_MEETING_VALUE)

    def test_ws_createItemWithExtraAttrs(self):
        """
          It is possible to specify arbitraty extraAttrs so we may create items even
          when using a specific profile adding is own fields.  For now it only works with
          XHTML TextFields.
        """
        self.changeUser('pmManager')
        req = self._prepareCreationData()
        ExtraAttr = GTD('http://ws4pm.imio.be', 'ExtraAttr')('').pyclass()
        ExtraAttr._key = 'unexisting_key'
        ExtraAttr._value = '<p>XHTML content</p>'
        req._creationData._extraAttrs = [ExtraAttr]
        responseHolder = createItemResponse()

        # key must be found in the MeetingItem's schema
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(cm.exception.string,
                         "The extraAttr 'unexisting_key' was not found the the MeetingItem schema!")

        # only works with RichText fields
        req._creationData._extraAttrs[0]._key = 'privacy'
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(
            cm.exception.string,
            "The extraAttr 'privacy' must correspond to a field using a 'RichWidget' in the MeetingItem schema!")

        # working example, use RichText field 'notes'
        req._creationData._extraAttrs[0]._key = 'notes'
        response = SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        item = self.portal.portal_catalog(UID=response._UID)[0].getObject()
        self.assertEqual(item.getNotes(), '<p>XHTML content</p>')

    def test_ws_createItemCleanHtml(self):
        """
          Special characters like &#xa0; in the html breaks PortalTransforms
          that is why we clean the HTML from wrong characters and styles
        """

        self.changeUser('pmCreator1')
        req = self._prepareCreationData()
        HTML = "<p style='padding-left: 5em'>My HTML</p><p>&#xa0;</p>"
        req._creationData._decision = HTML
        newItem, response = self._createItem(req)
        self.assertEqual(
            newItem.getDecision(keepWithNext=False),
            '<p>My HTML</p><p>\xc2\xa0</p>')

    def test_ws_createItemCleanHtmlEnabledDisabled(self):
        """
          It is possible to disable cleanHtml (enabled by default),
          this will not clean HTML data while creating the item.
        """
        # by default no item exists
        self.changeUser('pmCreator1')
        req = self._prepareCreationData()
        req._creationData._description = '<p style="font-size: 11pt">Description sample text</p>'
        # cleanHtml enabled
        self.assertEqual(req._cleanHtml, 1)
        newItem, response = self._createItem(req)
        self.assertEqual(newItem.Description(), '<p>Description sample text</p>')
        # cleanHtml disabled
        req._cleanHtml = 0
        newItem, response = self._createItem(req)
        self.assertEqual(newItem.Description(), '<p style="font-size: 11pt">Description sample text</p>')

    def test_ws_createItemGroupsInCharge(self):
        """
          Test when passing groupsInCharge while creating the item.
        """
        cfg = self.meetingConfig
        cfg.setUsedItemAttributes(['description', 'detailedDescription'])
        # by default no item exists
        self.changeUser('pmCreator1')
        req = self._prepareCreationData()

        # while passing no correct data
        req._creationData._groupsInCharge = [self.vendors_uid, 'unknown_uid']
        responseHolder = createItemResponse()
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        # optional field not enabled
        self.assertEqual(
            cm.exception.string,
            "The optional field \"groupsInCharge\" is not activated in this configuration!")
        # enable optional field, will fail because unknown_uid
        cfg.setUsedItemAttributes(cfg.getUsedItemAttributes() + ('groupsInCharge', ))

        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(
            cm.exception.string,
            'The \"groupsInCharge\" data contains wrong values: "unknown_uid"!')

        # now with correct data
        req._creationData._groupsInCharge = [self.vendors_uid, self.developers_uid]
        newItem, response = self._createItem(req)
        self.assertEqual(newItem.getGroupsInCharge(), [self.vendors_uid, self.developers_uid])

    def test_ws_createItemAssociatedGroups(self):
        """
          Test when passing associatedGroups while creating the item.
        """
        cfg = self.meetingConfig
        cfg.setUsedItemAttributes(['description', 'detailedDescription'])
        # by default no item exists
        self.changeUser('pmCreator1')
        req = self._prepareCreationData()

        # while passing no correct data
        req._creationData._associatedGroups = [self.vendors_uid, 'unknown_uid']
        responseHolder = createItemResponse()
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        # optional field not enabled
        self.assertEqual(
            cm.exception.string,
            "The optional field \"associatedGroups\" is not activated in this configuration!")
        # enable optional field, will fail because unknown_uid
        cfg.setUsedItemAttributes(cfg.getUsedItemAttributes() + ('associatedGroups', ))
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(
            cm.exception.string,
            'The \"associatedGroups\" data contains wrong values: "unknown_uid"!')

        # now with correct data
        req._creationData._associatedGroups = [self.vendors_uid, self.developers_uid]
        newItem, response = self._createItem(req)
        self.assertEqual(newItem.getAssociatedGroups(), (self.vendors_uid, self.developers_uid))

    def test_ws_createItemOptionalAdvisers(self):
        """
          Test when passing associatedGroups while creating the item.
        """
        cfg = self.meetingConfig
        cfg.setUsedItemAttributes(['description', 'detailedDescription'])

        # by default no item exists
        self.changeUser('pmCreator1')
        req = self._prepareCreationData()

        # while passing no correct data
        req._creationData._optionalAdvisers = [self.vendors_uid, 'unknown_uid']
        responseHolder = createItemResponse()
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        # advices not enabled
        self.assertEqual(
            cm.exception.string,
            "The advices functionnality is not enabled for this configuration!")
        # enable advices, will fail because unknown_uid
        cfg.setUseAdvices(True)
        cfg.setSelectableAdvisers([self.developers_uid, self.vendors_uid])
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).createItemRequest(req, responseHolder)
        self.assertEqual(
            cm.exception.string,
            'The \"optionalAdvisers\" data contains wrong values: "unknown_uid"!')

        # now with correct data
        req._creationData._optionalAdvisers = [self.developers_uid, self.vendors_uid]
        newItem, response = self._createItem(req)
        self.assertEqual(newItem.getOptionalAdvisers(), (self.developers_uid, self.vendors_uid))
        self.assertTrue(self.developers_uid in newItem.adviceIndex)
        self.assertTrue(self.vendors_uid in newItem.adviceIndex)

    def test_ws_createItemWfTransitions(self):
        """
          Test when passing wfTransitions while creating the item.
        """
        self.changeUser('pmCreator1')
        req = self._prepareCreationData()

        # correct wfTransitions
        req._wfTransitions = ['propose', 'validate']
        newItem, response = self._createItem(req)
        self.assertEqual(newItem.query_state(), 'validated')
        self.assertEqual(response._warnings, [])

        # correct and incorrect wfTransitions
        req._wfTransitions = ['propose', 'unknown_transition', 'validate']
        newItem, response = self._createItem(req)
        self.assertEqual(newItem.query_state(), 'validated')
        self.assertEqual(
            response._warnings,
            ["While treating wfTransitions, could not trigger the 'unknown_transition' transition!"])

        # 'present' with no available meeting
        req._wfTransitions = ['propose', 'validate', 'present']
        newItem, response = self._createItem(req)
        self.assertEqual(newItem.query_state(), 'validated')
        self.assertEqual(
            response._warnings,
            ["While treating wfTransitions, could not trigger the 'present' transition! "
             "Make sure a meeting accepting items exists in configuration 'plonegov-assembly'!"])

        # 'present' with available meeting
        self.changeUser('pmManager')
        meeting = self.create('Meeting')
        newItem, response = self._createItem(req)
        self.assertEqual(newItem.query_state(), 'presented')
        self.assertTrue(newItem in meeting.get_items())
        self.assertEqual(response._warnings, [])

    def test_ws_createItemWithAutoAskedAdvices(self):
        """
          Test item creation with auto asked advices.
          Test also the fact that advices are asked on item created from WS or not.
        """
        self.changeUser('siteadmin')
        cfg = self.meetingConfig
        # will ask advice auto when item created from WS
        cfg.setCustomAdvisers(
            [{'row_id': 'unique_id_123',
              'org': self.vendors_uid,
              'gives_auto_advice_on':
                "python: imio_history_utils.getLastWFAction(item, "
                "transition='create_item_using_imio_pm_ws_soap')",
              'for_item_created_from': '2016/08/08',
              'delay': '5',
              'delay_label': ''}, ])
        self.changeUser('pmCreator1')
        req = self._prepareCreationData()
        wsItem, response = self._createItem(req)
        self.assertTrue(wsItem.getAdviceDataFor(wsItem, self.vendors_uid))
        # now create an item manually, the auto advice is not asked
        item = self.create('MeetingItem')
        self.assertFalse(item.getAdviceDataFor(item, self.vendors_uid))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we
    # do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testSOAPCreateItem, prefix='test_ws_'))
    return suite
